@@ -6,9 +6,12 @@
  *
  * Written (W) 2013 Heiko Strathmann
  */
+
+#include <shogun/base/init.h>
 #include <shogun/lib/config.h>
 
 #include <shogun/kernel/string/DistantSegmentsKernel.h>
+#include <shogun/kernel/CustomKernel.h>
 #include <shogun/labels/BinaryLabels.h>
 #include <shogun/features/StringFeatures.h>
 #include <shogun/classifier/svm/SVMLight.h>
