@@ -11,14 +11,15 @@ int main(int argc, char** argv)
 	//sg_io->set_loglevel(MSG_DEBUG);
 	//sg_io->enable_file_and_line();
 
-	// create three 2-dimensional vectors 
+	// create three 2-dimensional vectors
 	SGMatrix<float64_t> matrix(2,3);
 	for (int32_t i=0; i<6; i++)
 		matrix.matrix[i]=i;
 
 	// shogun will now own the matrix created
 	CDenseFeatures<float64_t>* features= new CDenseFeatures<float64_t>(matrix);
-	volatile int ret = features->update_parameter_hash() ? 1 : 0;
+
+	ASSERT(features->parameter_hash_changed());
 
 	SG_UNREF(features);
 	exit_shogun();
