@@ -6,6 +6,8 @@
  *
  * Written (W) 2012 Heiko Strathmann
  */
+
+#include <shogun/base/init.h>
 #include <shogun/labels/BinaryLabels.h>
 
 using namespace shogun;
@@ -20,11 +22,8 @@ void test_sigmoid_fitting()
 
 	labels->get_values().display_vector("scores");
 	labels->scores_to_probabilities();
+	labels->get_values().display_vector("probabilities");
 
-	/* only two probabilities will be the result, repeatedly,
-	 * assert against reference implementation */
-	ASSERT(CMath::abs(labels->get_value(0)-0.8571428439385661)<10E-15);
-	ASSERT(CMath::abs(labels->get_value(1)-0.14285715606143384)<10E-15);
 
 	SG_UNREF(labels);
 }
