@@ -20,7 +20,7 @@
 #include <octave/Cell.h>
 #include <stdio.h>
 
-#include "ui/SGInterface.h"
+#include <shogun/ui/SGInterface.h>
 #include <shogun/lib/ShogunException.h>
 #include <shogun/io/SGIO.h>
 #include <shogun/lib/config.h>
@@ -260,7 +260,7 @@ void COctaveInterface::function_name(sg_type*& vec, int32_t& len)						\
 																					\
 	if (m.rows()!=1)																\
 		SG_ERROR("Expected " error_string " (1xN) Vector as argument %d, got vector " \
-			"of shape (%dx%d)\n", m_rhs_counter, m.rows(), m.cols()); 				\
+			"of shape (%dx%d)\n", m_rhs_counter, m.rows(), m.cols());				\
 																					\
 	len = m.cols();																	\
 	vec=SG_MALLOC(sg_type, len);															\
@@ -391,14 +391,14 @@ void COctaveInterface::function_name(SGString<sg_type>*& strings, int32_t& num_s
 			oct_type str=c.elem(i).oct_converter();									\
 																					\
 			int32_t len=str.cols();													\
-			if (len>0) 																\
-			{ 																		\
-				strings[i].slen=len; /* all must have same length in octave */ 		\
+			if (len>0)																\
+			{																		\
+				strings[i].slen=len; /* all must have same length in octave */		\
 				strings[i].string=SG_MALLOC(sg_type, len+1); /* not zero terminated in octave */ \
-				int32_t j; 															\
-				for (j=0; j<len; j++) 												\
-					strings[i].string[j]=str(0,j); 									\
-				strings[i].string[j]='\0'; 											\
+				int32_t j;															\
+				for (j=0; j<len; j++)												\
+					strings[i].string[j]=str(0,j);									\
+				strings[i].string[j]='\0';											\
 				max_string_len=CMath::max(max_string_len, len);						\
 			}																		\
 			else																	\
@@ -407,33 +407,33 @@ void COctaveInterface::function_name(SGString<sg_type>*& strings, int32_t& num_s
 				strings[i].slen=0;													\
 				strings[i].string=NULL;												\
 			}																		\
-		} 																			\
-	} 																				\
+		}																			\
+	}																				\
 	else if (arg.oct_type_check1() && arg.oct_type_check2())						\
 	{																				\
 		oct_type data=arg.oct_converter();											\
-		num_str=data.cols(); 														\
-		int32_t len=data.rows(); 													\
-		strings=SG_MALLOC(SGString<sg_type>, num_str); 								\
+		num_str=data.cols();														\
+		int32_t len=data.rows();													\
+		strings=SG_MALLOC(SGString<sg_type>, num_str);								\
 																					\
-		for (int32_t i=0; i<num_str; i++) 											\
-		{ 																			\
-			if (len>0) 																\
-			{ 																		\
-				strings[i].slen=len; /* all must have same length in octave */ 		\
+		for (int32_t i=0; i<num_str; i++)											\
+		{																			\
+			if (len>0)																\
+			{																		\
+				strings[i].slen=len; /* all must have same length in octave */		\
 				strings[i].string=SG_MALLOC(sg_type, len+1); /* not zero terminated in octave */ \
-				int32_t j; 															\
-				for (j=0; j<len; j++) 												\
+				int32_t j;															\
+				for (j=0; j<len; j++)												\
 					strings[i].string[j]=data(j,i);									\
-				strings[i].string[j]='\0'; 											\
-			} 																		\
-			else 																	\
-			{ 																		\
-				SG_WARNING( "string with index %d has zero length.\n", i+1); 		\
-				strings[i].slen=0; 													\
-				strings[i].string=NULL; 											\
-			} 																		\
-		} 																			\
+				strings[i].string[j]='\0';											\
+			}																		\
+			else																	\
+			{																		\
+				SG_WARNING( "string with index %d has zero length.\n", i+1);		\
+				strings[i].slen=0;													\
+				strings[i].string=NULL;											\
+			}																		\
+		}																			\
 		max_string_len=len;															\
 	}																				\
 	else																			\
@@ -545,7 +545,7 @@ void COctaveInterface::function_name(const SGString<sg_type>* strings, int32_t n
 	if (!strings)																	\
 		SG_ERROR("Given strings are invalid.\n")									\
 																					\
-	Cell c= Cell(dim_vector(num_str));												\
+	Cell c= Cell(dim_vector(1,num_str));											\
 	if (c.nelem()!=num_str)															\
 		SG_ERROR("Couldn't create Cell Array of %d strings.\n", num_str)			\
 																					\
@@ -601,14 +601,14 @@ bool COctaveInterface::cmd_run_r()
 
 void COctaveInterface::recover_from_exception(void)
 {
-#if OCTAVE_APIVERSION < 37
+#if defined(OCTAVE_APIVERSION) && OCTAVE_APIVERSION < 37
   unwind_protect::run_all ();
 #endif
   can_interrupt = true;
   octave_interrupt_immediately = 0;
   octave_interrupt_state = 0;
 
-#if OCTAVE_APIVERSION >= 37
+#if !defined(OCTAVE_APIVERSION) || OCTAVE_APIVERSION >= 37
   octave_exception_state = octave_no_exception;
 #else
   octave_allocation_error = 0;
@@ -633,8 +633,8 @@ void COctaveInterface::clear_octave_globals()
 
 void COctaveInterface::run_octave_init()
 {
-	char* name=strdup("octave");
-	char* opts=strdup("-q");
+	char* name=get_strdup("octave");
+	char* opts=get_strdup("-q");
 	char* argv[2]={name, opts};
 	octave_main(2,argv,1);
 	free(opts);
@@ -643,7 +643,11 @@ void COctaveInterface::run_octave_init()
 
 void COctaveInterface::run_octave_exit()
 {
+#if defined(OCTAVE_MAJOR_VERSION) && OCTAVE_MAJOR_VERSION >= 3 && defined(OCTAVE_MINOR_VERSION) && OCTAVE_MINOR_VERSION >= 8
+	clean_up_and_exit (0); 
+#else
 	do_octave_atexit();
+#endif
 }
 
 bool COctaveInterface::run_octave_helper(CSGInterface* from_if)
@@ -656,7 +660,7 @@ bool COctaveInterface::run_octave_helper(CSGInterface* from_if)
 #if defined (USE_EXCEPTIONS_FOR_INTERRUPTS)
 		panic_impossible ();
 #else
-#if OCTAVE_APIVERSION < 37
+#if defined(OCTAVE_APIVERSION) && OCTAVE_APIVERSION < 37
 		unwind_protect::run_all ();
 #endif
 		raw_mode (0);
@@ -668,7 +672,7 @@ bool COctaveInterface::run_octave_helper(CSGInterface* from_if)
 	octave_catch_interrupts ();
 	octave_initialized = true;
 
-	try 
+	try
 	{
 		int parse_status;
 		char* octave_code=NULL;
@@ -693,7 +697,7 @@ bool COctaveInterface::run_octave_helper(CSGInterface* from_if)
 				COctaveInterface* in = new COctaveInterface(args, 1, false);
 				in->create_return_values(1);
 				from_if->translate_arg(from_if, in);
-#if OCTAVE_APIVERSION >= 37
+#if !defined(OCTAVE_APIVERSION) || OCTAVE_APIVERSION >= 37
 				symbol_table::varref (var_name) = in->get_return_values()(0);
 #else
 				set_global_value(var_name, in->get_return_values()(0));
@@ -703,7 +707,7 @@ bool COctaveInterface::run_octave_helper(CSGInterface* from_if)
 			}
 		}
 
-#if OCTAVE_APIVERSION >= 37
+#if !defined(OCTAVE_APIVERSION) || OCTAVE_APIVERSION >= 37
 #else
 		symbol_table* old=curr_sym_tab;
 		curr_sym_tab = global_sym_tab;
@@ -715,7 +719,7 @@ bool COctaveInterface::run_octave_helper(CSGInterface* from_if)
 		int32_t sz=0;
 		octave_value_list results;
 
-#if OCTAVE_APIVERSION >= 37
+#if !defined(OCTAVE_APIVERSION) || OCTAVE_APIVERSION >= 37
 		if (symbol_table::is_variable("results"))
 		{
 			results = symbol_table::varval("results");
@@ -760,7 +764,7 @@ bool COctaveInterface::run_octave_helper(CSGInterface* from_if)
 			}
 		}
 
-#if OCTAVE_APIVERSION >= 37
+#if !defined(OCTAVE_APIVERSION) || OCTAVE_APIVERSION >= 37
 #else
 		curr_sym_tab=old;
 #endif
