@@ -14,10 +14,11 @@ extern "C" {
 
 #include <stdlib.h>
 #include <stdio.h>
-#include "ui/SGInterface.h"
+#include <shogun/ui/SGInterface.h>
 #include <shogun/lib/ShogunException.h>
 #include <shogun/io/SGIO.h>
 #include <shogun/base/init.h>
+#include <shogun/lib/memory.h>
 
 #ifdef HAVE_PYTHON
 #include "../python_static/PythonInterface.h"
@@ -552,8 +553,8 @@ void CRInterface::run_r_init()
 #ifdef R_HOME_ENV
 	setenv("R_HOME", R_HOME_ENV, 0);
 #endif
-	char* name=strdup("R");
-	char* opts=strdup("-q");
+	char* name=get_strdup("R");
+	char* opts=get_strdup("-q");
 	char* argv[2]={name, opts};
 	Rf_initEmbeddedR(2, argv);
 	free(opts);
@@ -603,7 +604,7 @@ bool CRInterface::run_r_helper(CSGInterface* from_if)
 		SG_OBJ_PRINT(from_if, "%s", e.get_exception_string())
 		return true;
 	}
-	
+
 	// Find source function
 	SEXP src = Rf_findFun(Rf_install("source"), R_GlobalEnv);
 	PROTECT(src);
@@ -616,7 +617,7 @@ bool CRInterface::run_r_helper(CSGInterface* from_if)
 	// expression source(file,print.eval=p)
 	SEXP expr;
 	PROTECT(expr = allocVector(LANGSXP,2));
-	SETCAR(expr,src); 
+	SETCAR(expr,src);
 	SETCAR(CDR(expr),file);
 
 	int err=0;
@@ -673,7 +674,7 @@ bool CRInterface::run_r_helper(CSGInterface* from_if)
 
 /* The main function of the shogun R interface. All commands from the R command line
  * to the shogun backend are passed using the syntax:
- * .External("sg", "func", ... ) 
+ * .External("sg", "func", ... )
  * where '...' is a number of arguments passed to the shogun function 'func'. */
 
 extern "C" {
@@ -698,11 +699,11 @@ void R_init_sg(DllInfo *info)
 
    R_CMethodDef cMethods[] = { {NULL, NULL, 0} };
    R_FortranMethodDef fortranMethods[] = { {NULL, NULL, 0} };
-   
+
 #ifdef HAVE_ELWMS
-   R_ExternalMethodDef externalMethods[] = { {"elwms", (void*(*)()) &Rsg, 1}, {NULL, NULL, 0} };
+   R_ExternalMethodDef externalMethods[] = { {"elwms", (void*(*)()) &Rsg, -1}, {NULL, NULL, 0} };
 #else
-   R_ExternalMethodDef externalMethods[] = { {"sg", (void*(*)()) &Rsg, 1}, {NULL, NULL, 0} };
+   R_ExternalMethodDef externalMethods[] = { {"sg", (void*(*)()) &Rsg, -1}, {NULL, NULL, 0} };
 #endif
    R_CallMethodDef callMethods[] = { {NULL, NULL, 0} };
 
@@ -713,7 +714,7 @@ void R_init_sg(DllInfo *info)
 
 SEXP Rsg(SEXP args)
 {
-	/* The SEXP (Simple Expression) args is a list of arguments of the .External call. 
+	/* The SEXP (Simple Expression) args is a list of arguments of the .External call.
 	 * it consists of "sg", "func" and additional arguments.
 	 * */
 
