@@ -5,21 +5,29 @@
  * (at your option) any later version.
  *
  * Written (W) 2010 Soeren Sonnenburg
+ * Written (W) 2011-2013 Heiko Strathmann
  * Copyright (C) 2010 Berlin Institute of Technology
  */
 #ifndef __PARAMETER_H__
 #define __PARAMETER_H__
 
+#include <shogun/lib/config.h>
+
 #include <shogun/lib/common.h>
-#include <shogun/io/SGIO.h>
 #include <shogun/lib/DataType.h>
-#include <shogun/lib/SGVector.h>
-#include <shogun/lib/SGMatrix.h>
-#include <shogun/io/SerializableFile.h>
 #include <shogun/base/DynArray.h>
 
 namespace shogun
 {
+
+class CSGObject;
+class CSerializableFile;
+template <class ST> class SGString;
+template <class T> class SGMatrix;
+template <class T> class SGSparseMatrix;
+template <class T> class SGVector;
+template <class T> class SGSparseVector;
+
 /** @brief parameter struct */
 struct TParameter
 {
@@ -75,6 +83,75 @@ struct TParameter
 	 * @param source source TParameter instance to copy from */
 	void copy_data(const TParameter* source);
 
+	/** Numerically this instance with another instance. Compares recursively
+	 * in case of non-numerical parameters
+	 *
+	 * @param other other instance to compare with
+	 * @param accuracy accuracy for numerical comparison
+	 * @param tolerant allows linient check on float equality (within accuracy)
+	 * @return true if given parameter instance is equal, false otherwise
+	 */
+	bool equals(TParameter* other, float64_t accuracy=0.0, bool tolerant=false);
+
+	/** Given two pointers to a scalar element of a given primitive-type, this
+	 * method compares the values up to a given accuracy.
+	 *
+	 * If the type of the data is SGObject, recursively calls equals on the
+	 * object.
+	 *
+	 * @param ptype primitive type of both data
+	 * @param data1 pointer 1
+	 * @param data2 pointer 2
+	 * @param accuracy accuracy to compare
+	 * @param tolerant allows linient check on float equality (within accuracy)
+	 * @return whether the data was equal
+	 */
+	static bool compare_ptype(EPrimitiveType ptype, void* data1, void* data2,
+			float64_t accuracy=0.0, bool tolerant=false);
+
+	/** Given two pointers to a string element of a given primitive-type, this
+	 * method compares the values up to a given accuracy.
+	 *
+	 * If the type of the data is SGObject, recursively calls equals on the
+	 * object.
+	 *
+	 * @param stype string type of both data
+	 * @param ptype primitive type of both data
+	 * @param data1 pointer 1
+	 * @param data2 pointer 2
+	 * @param accuracy accuracy to compare
+	 * @param tolerant allows linient check on float equality (within accuracy)
+	 * @return whether the data was equal
+	 */
+	static bool compare_stype(EStructType stype, EPrimitiveType ptype,
+			void* data1, void* data2, float64_t accuracy=0.0, bool tolerant=false);
+
+	/** copy primitive type from source to target
+	 *
+	 * @param ptype the primitive type
+	 * @param source from where to copy
+	 * @param target where to copy to
+	 */
+	static bool copy_ptype(EPrimitiveType ptype, void* source, void* target);
+
+	/** copy structured type from source to target
+	 *
+	 * @param stype the structured type
+	 * @param ptype the primitive type that the structured objects use
+	 * @param source from where to copy
+	 * @param target where to copy to
+	 */
+	static bool copy_stype(EStructType stype, EPrimitiveType ptype,
+				void* source, void* target);
+
+	/** copy this to parameter target
+	 *
+	 * @param target where this should be copied to
+	 */
+	bool copy(TParameter* target);
+
+
+
 	/** operator for comparison, (by string m_name) */
 	bool operator==(const TParameter& other) const;
 
@@ -331,6 +408,13 @@ public:
 	 * @param name name of parameter
 	 * @param description description of parameter
 	 */
+	void add(complex128_t* param, const char* name,
+			 const char* description="");
+	/** add param
+	 * @param param parameter itself
+	 * @param name name of parameter
+	 * @param description description of parameter
+	 */
 	void add(CSGObject** param,
 			 const char* name, const char* description="");
 	/** add param
@@ -515,6 +599,13 @@ public:
 	 */
 	void add(SGSparseVector<floatmax_t>* param, const char* name,
 			 const char* description="");
+	/** add param
+	 * @param param parameter itself
+	 * @param name name of parameter
+	 * @param description description of parameter
+	 */
+	void add(SGSparseVector<complex128_t>* param, const char* name,
+			 const char* description="");
 
 	/* ************************************************************ */
 	/* Vector wrappers  */
@@ -629,6 +720,14 @@ public:
 	 * @param name name of parameter
 	 * @param description description of parameter
 	 */
+	void add_vector(complex128_t** param, index_t* length,
+					const char* name, const char* description="");
+	/** add vector param
+	 * @param param parameter vector itself
+	 * @param length length of vector
+	 * @param name name of parameter
+	 * @param description description of parameter
+	 */
 	void add_vector(CSGObject*** param, index_t* length,
 					const char* name, const char* description="");
 	/** add vector param
@@ -839,6 +938,14 @@ public:
 	 */
 	void add_vector(SGSparseVector<floatmax_t>** param, index_t* length,
 					const char* name, const char* description="");
+	/** add vector param
+	 * @param param parameter vector itself
+	 * @param length length of vector
+	 * @param name name of parameter
+	 * @param description description of parameter
+	 */
+	void add_vector(SGSparseVector<complex128_t>** param, index_t* length,
+					const char* name, const char* description="");
 
 
 	/** add vector param
@@ -937,6 +1044,13 @@ public:
 	 * @param name name of parameter
 	 * @param description description of parameter
 	 */
+	void add(SGVector<complex128_t>* param, const char* name,
+					const char* description="");
+	/** add vector param
+	 * @param param parameter vector itself
+	 * @param name name of parameter
+	 * @param description description of parameter
+	 */
 	void add(SGVector<CSGObject*>* param, const char* name,
 					const char* description="");
 	/** add vector param
@@ -1121,6 +1235,13 @@ public:
 	 */
 	void add(SGVector<SGSparseVector<floatmax_t> >* param,
 					const char* name, const char* description="");
+	/** add vector param
+	 * @param param parameter vector itself
+	 * @param name name of parameter
+	 * @param description description of parameter
+	 */
+	void add(SGVector<SGSparseVector<complex128_t> >* param,
+					const char* name, const char* description="");
 
 	/* ************************************************************ */
 	/* Matrix wrappers  */
@@ -1262,6 +1383,16 @@ public:
 	 * @param name name of parameter
 	 * @param description description of parameter
 	 */
+	void add_matrix(complex128_t** param,
+					index_t* length_y, index_t* length_x,
+					const char* name, const char* description="");
+	/** add matrix param
+	 * @param param parameter matrix itself
+	 * @param length_y y size of matrix
+	 * @param length_x x size of matrix
+	 * @param name name of parameter
+	 * @param description description of parameter
+	 */
 	void add_matrix(CSGObject*** param,
 					index_t* length_y, index_t* length_x,
 					const char* name, const char* description="");
@@ -1525,6 +1656,16 @@ public:
 	void add_matrix(SGSparseVector<floatmax_t>** param,
 					index_t* length_y, index_t* length_x,
 					const char* name, const char* description="");
+	/** add matrix param
+	 * @param param parameter matrix itself
+	 * @param length_y y size of matrix
+	 * @param length_x x size of matrix
+	 * @param name name of parameter
+	 * @param description description of parameter
+	 */
+	void add_matrix(SGSparseVector<complex128_t>** param,
+					index_t* length_y, index_t* length_x,
+					const char* name, const char* description="");
 	/** add matrix param
 	 * @param param parameter matrix itself
 	 * @param name name of parameter
@@ -1621,6 +1762,13 @@ public:
 	 * @param name name of parameter
 	 * @param description description of parameter
 	 */
+	void add(SGMatrix<complex128_t>* param, const char* name,
+					const char* description="");
+	/** add matrix param
+	 * @param param parameter matrix itself
+	 * @param name name of parameter
+	 * @param description description of parameter
+	 */
 	void add(SGMatrix<CSGObject*>* param, const char* name,
 					const char* description="");
 	/** add matrix param
@@ -1805,7 +1953,119 @@ public:
 	 */
 	void add(SGMatrix<SGSparseVector<floatmax_t> >* param,
 					const char* name, const char* description="");
+	/** add matrix param
+	 * @param param parameter matrix itself
+	 * @param name name of parameter
+	 * @param description description of parameter
+	 */
+	void add(SGMatrix<SGSparseVector<complex128_t> >* param,
+					const char* name, const char* description="");
 
+	/** add matrix param
+	 * @param param parameter matrix itself
+	 * @param name name of parameter
+	 * @param description description of parameter
+	 */
+	void add(SGSparseMatrix<bool>* param,
+					const char* name, const char* description="");
+	/** add matrix param
+	 * @param param parameter matrix itself
+	 * @param name name of parameter
+	 * @param description description of parameter
+	 */
+	void add(SGSparseMatrix<char>* param,
+					const char* name, const char* description="");
+	/** add matrix param
+	 * @param param parameter matrix itself
+	 * @param name name of parameter
+	 * @param description description of parameter
+	 */
+	void add(SGSparseMatrix<int8_t>* param,
+					const char* name, const char* description="");
+	/** add matrix param
+	 * @param param parameter matrix itself
+	 * @param name name of parameter
+	 * @param description description of parameter
+	 */
+	void add(SGSparseMatrix<uint8_t>* param,
+					const char* name, const char* description="");
+	/** add matrix param
+	 * @param param parameter matrix itself
+	 * @param name name of parameter
+	 * @param description description of parameter
+	 */
+	void add(SGSparseMatrix<int16_t>* param,
+					const char* name, const char* description="");
+	/** add matrix param
+	 * @param param parameter matrix itself
+	 * @param name name of parameter
+	 * @param description description of parameter
+	 */
+	void add(SGSparseMatrix<uint16_t>* param,
+					const char* name, const char* description="");
+	/** add matrix param
+	 * @param param parameter matrix itself
+	 * @param name name of parameter
+	 * @param description description of parameter
+	 */
+	void add(SGSparseMatrix<int32_t>* param,
+					const char* name, const char* description="");
+	/** add matrix param
+	 * @param param parameter matrix itself
+	 * @param name name of parameter
+	 * @param description description of parameter
+	 */
+	void add(SGSparseMatrix<uint32_t>* param,
+					const char* name, const char* description="");
+	/** add matrix param
+	 * @param param parameter matrix itself
+	 * @param name name of parameter
+	 * @param description description of parameter
+	 */
+	void add(SGSparseMatrix<int64_t>* param,
+					const char* name, const char* description="");
+	/** add matrix param
+	 * @param param parameter matrix itself
+	 * @param name name of parameter
+	 * @param description description of parameter
+	 */
+	void add(SGSparseMatrix<uint64_t>* param,
+					const char* name, const char* description="");
+	/** add matrix param
+	 * @param param parameter matrix itself
+	 * @param name name of parameter
+	 * @param description description of parameter
+	 */
+	void add(SGSparseMatrix<float32_t>* param,
+					const char* name, const char* description="");
+	/** add matrix param
+	 * @param param parameter matrix itself
+	 * @param name name of parameter
+	 * @param description description of parameter
+	 */
+	void add(SGSparseMatrix<float64_t>* param,
+					const char* name, const char* description="");
+	/** add matrix param
+	 * @param param parameter matrix itself
+	 * @param name name of parameter
+	 * @param description description of parameter
+	 */
+	void add(SGSparseMatrix<floatmax_t>* param,
+					const char* name, const char* description="");
+	/** add matrix param
+	 * @param param parameter matrix itself
+	 * @param name name of parameter
+	 * @param description description of parameter
+	 */
+	void add(SGSparseMatrix<complex128_t>* param,
+					const char* name, const char* description="");
+	/** add matrix param
+	 * @param param parameter matrix itself
+	 * @param name name of parameter
+	 * @param description description of parameter
+	 */
+	void add(SGSparseMatrix<CSGObject*>* param,
+					const char* name, const char* description="");
 protected:
 
 	/** array of parameters */
