@@ -5,33 +5,35 @@
  * (at your option) any later version.
  *
  * Written (W) 2008-2009 Soeren Sonnenburg
- * Written (W) 2011-2012 Heiko Strathmann
+ * Written (W) 2011-2013 Heiko Strathmann
+ * Written (W) 2013-2014 Thoralf Klein
  * Copyright (C) 2008-2009 Fraunhofer Institute FIRST and Max Planck Society
  */
 
 #include <shogun/lib/config.h>
+#include <shogun/lib/memory.h>
+#include <shogun/lib/RefCount.h>
+
 #include <shogun/base/SGObject.h>
 #include <shogun/io/SGIO.h>
-#include <shogun/base/Parallel.h>
-#include <shogun/base/init.h>
 #include <shogun/base/Version.h>
 #include <shogun/base/Parameter.h>
 #include <shogun/base/ParameterMap.h>
 #include <shogun/base/DynArray.h>
 #include <shogun/lib/Map.h>
+#include <shogun/lib/SGVector.h>
+#include <shogun/lib/SGStringList.h>
+#include <shogun/io/SerializableFile.h>
 
-
+#include <shogun/base/class_list.h>
 
 #include <stdlib.h>
 #include <stdio.h>
+#include <string.h>
 
-#include <shogun/features/DenseFeatures.h>
 namespace shogun
 {
-	class CMath;
 	class Parallel;
-	class IO;
-	class Version;
 
 	extern Parallel* sg_parallel;
 	extern SGIO* sg_io;
@@ -102,6 +104,16 @@ namespace shogun
 		m_generic = PT_FLOATMAX;
 	}
 
+	template<> void CSGObject::set_generic<CSGObject*>()
+	{
+		m_generic = PT_SGOBJECT;
+	}
+
+	template<> void CSGObject::set_generic<complex128_t>()
+	{
+		m_generic = PT_COMPLEX128;
+	}
+
 } /* namespace shogun  */
 
 using namespace shogun;
@@ -110,8 +122,9 @@ CSGObject::CSGObject()
 {
 	init();
 	set_global_objects();
+	m_refcount = new RefCount(0);
 
-	SG_GCDEBUG("SGObject created (%p)\n", this)
+	SG_SGCDEBUG("SGObject created (%p)\n", this)
 }
 
 CSGObject::CSGObject(const CSGObject& orig)
@@ -119,75 +132,76 @@ CSGObject::CSGObject(const CSGObject& orig)
 {
 	init();
 	set_global_objects();
+	m_refcount = new RefCount(0);
+
+	SG_SGCDEBUG("SGObject copied (%p)\n", this)
 }
 
 CSGObject::~CSGObject()
 {
-	SG_GCDEBUG("SGObject destroyed (%p)\n", this)
+	SG_SGCDEBUG("SGObject destroyed (%p)\n", this)
 
-#ifdef HAVE_PTHREAD
-	PTHREAD_LOCK_DESTROY(&m_ref_lock);
-#endif
 	unset_global_objects();
 	delete m_parameters;
 	delete m_model_selection_parameters;
+	delete m_gradient_parameters;
 	delete m_parameter_map;
+	delete m_refcount;
 }
 
 #ifdef USE_REFERENCE_COUNTING
-
 int32_t CSGObject::ref()
 {
-#ifdef HAVE_PTHREAD
-		PTHREAD_LOCK(&m_ref_lock);
-#endif //HAVE_PTHREAD
-		++m_refcount;
-		int32_t count=m_refcount;
-#ifdef HAVE_PTHREAD
-		PTHREAD_UNLOCK(&m_ref_lock);
-#endif //HAVE_PTHREAD
-		SG_GCDEBUG("ref() refcount %ld obj %s (%p) increased\n", count, this->get_name(), this)
-		return m_refcount;
+	int32_t count = m_refcount->ref();
+	SG_SGCDEBUG("ref() refcount %ld obj %s (%p) increased\n", count, this->get_name(), this)
+	return m_refcount->ref_count();
 }
 
 int32_t CSGObject::ref_count()
 {
-#ifdef HAVE_PTHREAD
-	PTHREAD_LOCK(&m_ref_lock);
-#endif //HAVE_PTHREAD
-	int32_t count=m_refcount;
-#ifdef HAVE_PTHREAD
-	PTHREAD_UNLOCK(&m_ref_lock);
-#endif //HAVE_PTHREAD
-	SG_GCDEBUG("ref_count(): refcount %d, obj %s (%p)\n", count, this->get_name(), this)
-	return count;
+	int32_t count = m_refcount->ref_count();
+	SG_SGCDEBUG("ref_count(): refcount %d, obj %s (%p)\n", count, this->get_name(), this)
+	return m_refcount->ref_count();
 }
 
 int32_t CSGObject::unref()
 {
-#ifdef HAVE_PTHREAD
-	PTHREAD_LOCK(&m_ref_lock);
-#endif //HAVE_PTHREAD
-	if (m_refcount==0 || --m_refcount==0)
+	int32_t count = m_refcount->unref();
+	if (count<=0)
 	{
-		SG_GCDEBUG("unref() refcount %ld, obj %s (%p) destroying\n", m_refcount, this->get_name(), this)
-#ifdef HAVE_PTHREAD
-		PTHREAD_UNLOCK(&m_ref_lock);
-#endif //HAVE_PTHREAD
+		SG_SGCDEBUG("unref() refcount %ld, obj %s (%p) destroying\n", count, this->get_name(), this)
 		delete this;
 		return 0;
 	}
 	else
 	{
-		SG_GCDEBUG("unref() refcount %ld obj %s (%p) decreased\n", m_refcount, this->get_name(), this)
-#ifdef HAVE_PTHREAD
-		PTHREAD_UNLOCK(&m_ref_lock);
-#endif //HAVE_PTHREAD
-		return m_refcount;
+		SG_SGCDEBUG("unref() refcount %ld obj %s (%p) decreased\n", count, this->get_name(), this)
+		return m_refcount->ref_count();
 	}
 }
 #endif //USE_REFERENCE_COUNTING
 
+#ifdef TRACE_MEMORY_ALLOCS
+#include <shogun/lib/Map.h>
+extern CMap<void*, shogun::MemoryBlock>* sg_mallocs;
+
+void CSGObject::list_memory_allocs()
+{
+	shogun::list_memory_allocs();
+}
+#endif
+
+CSGObject * CSGObject::shallow_copy() const
+{
+	SG_NOTIMPLEMENTED
+	return NULL;
+}
+
+CSGObject * CSGObject::deep_copy() const
+{
+	SG_NOTIMPLEMENTED
+	return NULL;
+}
 
 void CSGObject::set_global_objects()
 {
@@ -215,9 +229,9 @@ void CSGObject::unset_global_objects()
 
 void CSGObject::set_global_io(SGIO* new_io)
 {
+	SG_REF(new_io);
 	SG_UNREF(sg_io);
 	sg_io=new_io;
-	SG_REF(sg_io);
 }
 
 SGIO* CSGObject::get_global_io()
@@ -228,31 +242,37 @@ SGIO* CSGObject::get_global_io()
 
 void CSGObject::set_global_parallel(Parallel* new_parallel)
 {
+	SG_REF(new_parallel);
 	SG_UNREF(sg_parallel);
 	sg_parallel=new_parallel;
-	SG_REF(sg_parallel);
 }
 
-bool CSGObject::update_parameter_hash()
+void CSGObject::update_parameter_hash()
 {
-	uint32_t new_hash = 0;
-	uint32_t carry = 0;
-	uint32_t length = 0;
+	SG_DEBUG("entering\n")
 
-	get_parameter_incremental_hash(m_parameters, new_hash,
-			carry, length);
+	uint32_t carry=0;
+	uint32_t length=0;
 
-	new_hash = CHash::FinalizeIncrementalMurmurHash3(new_hash,
-			carry, length);
+	get_parameter_incremental_hash(m_hash, carry, length);
+	m_hash=CHash::FinalizeIncrementalMurmurHash3(m_hash, carry, length);
 
-	if(new_hash != m_hash)
-	{
-		m_hash = new_hash;
-		return true;
-	}
+	SG_DEBUG("leaving\n")
+}
 
-	else
-		return false;
+bool CSGObject::parameter_hash_changed()
+{
+	SG_DEBUG("entering\n")
+
+	uint32_t hash=0;
+	uint32_t carry=0;
+	uint32_t length=0;
+
+	get_parameter_incremental_hash(hash, carry, length);
+	hash=CHash::FinalizeIncrementalMurmurHash3(hash, carry, length);
+
+	SG_DEBUG("leaving\n")
+	return (m_hash!=hash);
 }
 
 Parallel* CSGObject::get_global_parallel()
@@ -263,9 +283,9 @@ Parallel* CSGObject::get_global_parallel()
 
 void CSGObject::set_global_version(Version* new_version)
 {
+	SG_REF(new_version);
 	SG_UNREF(sg_version);
 	sg_version=new_version;
-	SG_REF(sg_version);
 }
 
 Version* CSGObject::get_global_version()
@@ -311,7 +331,7 @@ bool CSGObject::save_serializable(CSerializableFile* file,
 	if (!m_save_pre_called)
 	{
 		SG_SWARNING("%s%s::save_serializable_pre(): Implementation "
-				   "error: BASE_CLASS::LOAD_SERIALIZABLE_PRE() not "
+				   "error: BASE_CLASS::SAVE_SERIALIZABLE_PRE() not "
 				   "called!\n", prefix, get_name());
 		return false;
 	}
@@ -338,7 +358,7 @@ bool CSGObject::save_serializable(CSerializableFile* file,
 	if (!m_save_post_called)
 	{
 		SG_SWARNING("%s%s::save_serializable_post(): Implementation "
-				   "error: BASE_CLASS::LOAD_SERIALIZABLE_POST() not "
+				   "error: BASE_CLASS::SAVE_SERIALIZABLE_POST() not "
 				   "called!\n", prefix, get_name());
 		return false;
 	}
@@ -354,6 +374,8 @@ bool CSGObject::save_serializable(CSerializableFile* file,
 bool CSGObject::load_serializable(CSerializableFile* file,
 		const char* prefix, int32_t param_version)
 {
+	REQUIRE(file != NULL, "Serializable file object should be != NULL\n");
+
 	SG_DEBUG("START LOADING CSGObject '%s'\n", get_name())
 	try
 	{
@@ -389,7 +411,7 @@ bool CSGObject::load_serializable(CSerializableFile* file,
 
 	if (file_version>param_version)
 	{
-		if (param_version==VERSION_PARAMETER)
+		if (param_version==Version::get_version_parameter())
 		{
 			SG_WARNING("%s%s::load_serializable(): parameter version of file "
 					"larger than the one of shogun. Try with a more recent"
@@ -1021,6 +1043,8 @@ bool CSGObject::save_parameter_version(CSerializableFile* file,
 int32_t CSGObject::load_parameter_version(CSerializableFile* file,
 		const char* prefix)
 {
+	REQUIRE(file != NULL, "Serializable file object should be != NULL");
+
 	TSGDataType t(CT_SCALAR, ST_NONE, PT_INT32);
 	int32_t v;
 	TParameter tp(&t, &v, "version_parameter", "");
@@ -1057,10 +1081,6 @@ extern CMap<void*, shogun::MemoryBlock>* sg_mallocs;
 
 void CSGObject::init()
 {
-#ifdef HAVE_PTHREAD
-	PTHREAD_LOCK_INIT(&m_ref_lock);
-#endif
-
 #ifdef TRACE_MEMORY_ALLOCS
 	if (sg_mallocs)
 	{
@@ -1073,16 +1093,18 @@ void CSGObject::init()
 	}
 #endif
 
-	m_refcount = 0;
 	io = NULL;
 	parallel = NULL;
 	version = NULL;
 	m_parameters = new Parameter();
 	m_model_selection_parameters = new Parameter();
+	m_gradient_parameters=new Parameter();
 	m_parameter_map=new ParameterMap();
 	m_generic = PT_NOT_GENERIC;
 	m_load_pre_called = false;
 	m_load_post_called = false;
+	m_save_pre_called = false;
+	m_save_post_called = false;
 	m_hash = 0;
 }
 
@@ -1177,55 +1199,191 @@ bool CSGObject::is_param_new(const SGParamInfo param_info) const
 	return result;
 }
 
-void CSGObject::get_parameter_incremental_hash(Parameter* param,
-		uint32_t& hash, uint32_t& carry, uint32_t& total_length)
+void CSGObject::get_parameter_incremental_hash(uint32_t& hash, uint32_t& carry,
+		uint32_t& total_length)
 {
-	if (!param)
-		return;
-
-	for (index_t i=0; i<param->get_num_parameters(); i++)
+	for (index_t i=0; i<m_parameters->get_num_parameters(); i++)
 	{
-		TParameter* p = param->get_parameter(i);
+		TParameter* p=m_parameters->get_parameter(i);
 
-		if (!p || !p->is_valid())
-			continue;
+		SG_DEBUG("Updating hash for parameter %s.%s\n", get_name(), p->m_name);
 
-		if (p->m_datatype.m_ptype != PT_SGOBJECT)
+		if (p->m_datatype.m_ptype == PT_SGOBJECT)
 		{
-			p->get_incremental_hash(hash, carry, total_length);
-			continue;
-		}
-
-		CSGObject* child = *((CSGObject**)(p->m_parameter));
-
-		if (child)
-			get_parameter_incremental_hash(
-					child->m_parameters, hash,
-					carry, total_length);
-	}
-}
-
-void CSGObject::build_parameter_dictionary(CMap<TParameter*, CSGObject*>& dict)
-{
-	if (m_parameters)
-	{
-		for (index_t i=0; i<m_parameters->get_num_parameters(); i++)
-		{
-			TParameter* p = m_parameters->get_parameter(i);
-
-			dict.add(p, this);
-
-			if ((p->m_datatype.m_ptype == PT_SGOBJECT) &&
-					(p->m_datatype.m_ctype == CT_SCALAR) &&
-					(*(CSGObject**)(p->m_parameter) != NULL))
+			if (p->m_datatype.m_ctype == CT_SCALAR)
 			{
-				CSGObject* child =
-						*((CSGObject**)(p->m_parameter));
-				if (child)
-					child->build_parameter_dictionary(dict);
-			}
+				CSGObject* child = *((CSGObject**)(p->m_parameter));
 
+				if (child)
+				{
+					child->get_parameter_incremental_hash(hash, carry,
+							total_length);
+				}
+			}
+			else if (p->m_datatype.m_ctype==CT_VECTOR ||
+					p->m_datatype.m_ctype==CT_SGVECTOR)
+			{
+				CSGObject** child=(*(CSGObject***)(p->m_parameter));
+
+				for (index_t j=0; j<*(p->m_datatype.m_length_y); j++)
+				{
+					if (child[j])
+					{
+						child[j]->get_parameter_incremental_hash(hash, carry,
+								total_length);
+					}
+				}
+			}
+		}
+		else
+			p->get_incremental_hash(hash, carry, total_length);
+	}
+}
+
+void CSGObject::build_gradient_parameter_dictionary(CMap<TParameter*, CSGObject*>* dict)
+{
+	for (index_t i=0; i<m_gradient_parameters->get_num_parameters(); i++)
+	{
+		TParameter* p=m_gradient_parameters->get_parameter(i);
+		dict->add(p, this);
+	}
+
+	for (index_t i=0; i<m_model_selection_parameters->get_num_parameters(); i++)
+	{
+		TParameter* p=m_model_selection_parameters->get_parameter(i);
+		CSGObject* child=*(CSGObject**)(p->m_parameter);
+
+		if ((p->m_datatype.m_ptype == PT_SGOBJECT) &&
+				(p->m_datatype.m_ctype == CT_SCALAR) &&	child)
+		{
+			child->build_gradient_parameter_dictionary(dict);
 		}
 	}
 }
 
+bool CSGObject::equals(CSGObject* other, float64_t accuracy, bool tolerant)
+{
+	SG_DEBUG("entering %s::equals()\n", get_name());
+
+	if (other==this)
+	{
+		SG_DEBUG("leaving %s::equals(): other object is me\n", get_name());
+		return true;
+	}
+
+	if (!other)
+	{
+		SG_DEBUG("leaving %s::equals(): other object is NULL\n", get_name());
+		return false;
+	}
+
+	SG_DEBUG("comparing \"%s\" to \"%s\"\n", get_name(), other->get_name());
+
+	/* a crude type check based on the get_name */
+	if (strcmp(other->get_name(), get_name()))
+	{
+		SG_INFO("leaving %s::equals(): name of other object differs\n", get_name());
+		return false;
+	}
+
+	/* should not be necessary but just ot be sure that type has not changed.
+	 * Will assume that parameters are in same order with same name from here */
+	if (m_parameters->get_num_parameters()!=other->m_parameters->get_num_parameters())
+	{
+		SG_INFO("leaving %s::equals(): number of parameters of other object "
+				"differs\n", get_name());
+		return false;
+	}
+
+	for (index_t i=0; i<m_parameters->get_num_parameters(); ++i)
+	{
+		SG_DEBUG("comparing parameter %d\n", i);
+
+		TParameter* this_param=m_parameters->get_parameter(i);
+		TParameter* other_param=other->m_parameters->get_parameter(i);
+
+		/* some checks to make sure parameters have same order and names and
+		 * are not NULL. Should never be the case but check anyway. */
+		if (!this_param && !other_param)
+			continue;
+
+		if (!this_param && other_param)
+		{
+			SG_DEBUG("leaving %s::equals(): parameter %d is NULL where other's "
+					"parameter \"%s\" is not\n", get_name(), other_param->m_name);
+			return false;
+		}
+
+		if (this_param && !other_param)
+		{
+			SG_DEBUG("leaving %s::equals(): parameter %d is \"%s\" where other's "
+						"parameter is NULL\n", get_name(), this_param->m_name);
+			return false;
+		}
+
+		SG_DEBUG("comparing parameter \"%s\" to other's \"%s\"\n",
+				this_param->m_name, other_param->m_name);
+
+		/* hard-wired exception for DynamicObjectArray parameter num_elements */
+		if (!strcmp("DynamicObjectArray", get_name()) &&
+				!strcmp(this_param->m_name, "num_elements") &&
+				!strcmp(other_param->m_name, "num_elements"))
+		{
+			SG_DEBUG("Ignoring DynamicObjectArray::num_elements field\n");
+			continue;
+		}
+
+		/* hard-wired exception for DynamicArray parameter num_elements */
+		if (!strcmp("DynamicArray", get_name()) &&
+				!strcmp(this_param->m_name, "num_elements") &&
+				!strcmp(other_param->m_name, "num_elements"))
+		{
+			SG_DEBUG("Ignoring DynamicArray::num_elements field\n");
+			continue;
+		}
+
+		/* use equals method of TParameter from here */
+		if (!this_param->equals(other_param, accuracy, tolerant))
+		{
+			SG_INFO("leaving %s::equals(): parameters at position %d with name"
+					" \"%s\" differs from other object parameter with name "
+					"\"%s\"\n",
+					get_name(), i, this_param->m_name, other_param->m_name);
+			return false;
+		}
+	}
+
+	SG_DEBUG("leaving %s::equals(): object are equal\n", get_name());
+	return true;
+}
+
+CSGObject* CSGObject::clone()
+{
+	SG_DEBUG("entering %s::clone()\n", get_name());
+
+	SG_DEBUG("constructing an empty instance of %s\n", get_name());
+	CSGObject* copy=new_sgserializable(get_name(), this->m_generic);
+
+	SG_REF(copy);
+
+	REQUIRE(copy, "Could not create empty instance of \"%s\". The reason for "
+			"this usually is that get_name() of the class returns something "
+			"wrong, or that a class has a wrongly set generic type.\n",
+			get_name());
+
+	for (index_t i=0; i<m_parameters->get_num_parameters(); ++i)
+	{
+		SG_DEBUG("cloning parameter \"%s\" at index %d\n",
+				m_parameters->get_parameter(i)->m_name, i);
+
+		if (!m_parameters->get_parameter(i)->copy(copy->m_parameters->get_parameter(i)))
+		{
+			SG_DEBUG("leaving %s::clone(): Clone failed. Returning NULL\n",
+					get_name());
+			return NULL;
+		}
+	}
+
+	SG_DEBUG("leaving %s::clone(): Clone successful\n", get_name());
+	return copy;
+}
