@@ -10,6 +10,7 @@
 #ifndef __MKLCLASSIFICATION_H__
 #define __MKLCLASSIFICATION_H__
 
+#include <shogun/lib/config.h>
 #include <shogun/lib/common.h>
 #include <shogun/classifier/mkl/MKL.h>
 
@@ -40,6 +41,9 @@ class CMKLClassification : public CMKL
 		 */
 		virtual float64_t compute_sum_alpha();
 
+		/** @return object name */
+		virtual const char* get_name() const { return "MKLClassification"; }
+
 	protected:
 		/** check run before starting training (to e.g. check if labeling is
 		 * two-class labeling in classification case
