@@ -10,6 +10,8 @@
 #ifndef __MKLONECLASS_H__
 #define __MKLONECLASS_H__
 
+#include <shogun/lib/config.h>
+
 #include <shogun/lib/common.h>
 #include <shogun/classifier/mkl/MKL.h>
 
@@ -40,6 +42,9 @@ class CMKLOneClass : public CMKL
 		 */
 		virtual float64_t compute_sum_alpha();
 
+		/** @return object name */
+		virtual const char* get_name() const { return "MKLOneClass"; }
+
 	protected:
 		/** check run before starting training (to e.g. check if labeling is
 		 * two-class labeling in classification case
