@@ -5,6 +5,7 @@
  * (at your option) any later version.
  *
  * Written (W) 1999-2009 Soeren Sonnenburg
+ * Written (W) 2014 Parijat Mazumdar
  * Copyright (C) 1999-2009 Fraunhofer Institute FIRST and Max-Planck-Society
  */
 
@@ -21,6 +22,7 @@ CDistribution::CDistribution()
 
 CDistribution::~CDistribution()
 {
+	SG_UNREF(features);
 }
 
 float64_t CDistribution::get_log_likelihood_sample()
@@ -59,3 +61,35 @@ int32_t CDistribution::get_num_relevant_model_parameters()
 	}
 	return num;
 }
+
+SGVector<float64_t> CDistribution::get_likelihood_for_all_examples()
+{
+	ASSERT(features);
+	int32_t num=features->get_num_vectors();
+	ASSERT(num>0);
+
+	SGVector<float64_t> result=SGVector<float64_t>(num);
+	for (int32_t i=0; i<num; i++)
+		result[i]=get_likelihood_example(i);
+
+	return result;
+}
+
+float64_t CDistribution::update_params_em(float64_t* alpha_k, int32_t len)
+{
+	SG_WARNING("Not implemented in this class. This class cannot be used for Mixture models.\n")
+	SG_NOTIMPLEMENTED
+	return -1;
+}
+
+CDistribution* CDistribution::obtain_from_generic(CSGObject* object)
+{
+	if (!object)
+		return NULL;
+
+	CDistribution* casted=dynamic_cast<CDistribution*>(object);
+	if (!casted)
+		return NULL;
+
+	return casted;
+}
\ No newline at end of file
