@@ -55,7 +55,7 @@ template<class ST> CDenseFeatures<ST>::CDenseFeatures(ST* src, int32_t num_feat,
 	set_feature_matrix(SGMatrix<ST>(src, num_feat, num_vec));
 }
 template<class ST> CDenseFeatures<ST>::CDenseFeatures(CFile* loader) :
-		CDotFeatures(loader)
+		CDotFeatures()
 {
 	init();
 	load(loader);
@@ -137,10 +137,14 @@ template<class ST> ST* CDenseFeatures<ST>::get_feature_vector(int32_t num, int32
 			tmp_feat_before = tmp_feat_after;
 		}
 
-		memcpy(feat, tmp_feat_after, sizeof(ST) * tmp_len);
-		SG_FREE(tmp_feat_after);
+		// note: tmp_feat_after should be checked as it is used by memcpy
+		if (tmp_feat_after)
+		{
+			memcpy(feat, tmp_feat_after, sizeof(ST) * tmp_len);
+			SG_FREE(tmp_feat_after);
 
-		len = tmp_len;
+			len = tmp_len;
+		}
 	}
 	return feat;
 }
@@ -414,14 +418,12 @@ template<class ST> bool CDenseFeatures<ST>::apply_preprocessor(bool force_prepro
 	}
 }
 
-template<class ST> int32_t CDenseFeatures<ST>::get_size() const  { return sizeof(ST); }
-
 template<class ST> int32_t CDenseFeatures<ST>::get_num_vectors() const
 {
 	return m_subset_stack->has_subsets() ? m_subset_stack->get_size() : num_vectors;
 }
 
-template<class ST> int32_t CDenseFeatures<ST>::get_num_features() { return num_features; }
+template<class ST> int32_t CDenseFeatures<ST>::get_num_features() const { return num_features; }
 
 template<class ST> void CDenseFeatures<ST>::set_num_features(int32_t num)
 {
@@ -485,7 +487,7 @@ template<class ST> float64_t CDenseFeatures<ST>::dot(int32_t vec_idx1, CDotFeatu
 	ST* vec1 = get_feature_vector(vec_idx1, len1, free1);
 	ST* vec2 = sf->get_feature_vector(vec_idx2, len2, free2);
 
-	float64_t result = SGVector<ST>::dot(vec1, vec2, len1);
+	float64_t result = CMath::dot(vec1, vec2, len1);
 
 	free_feature_vector(vec1, vec_idx1, free1);
 	sf->free_feature_vector(vec2, vec_idx2, free2);
@@ -604,6 +606,36 @@ template<class ST> CFeatures* CDenseFeatures<ST>::copy_subset(SGVector<index_t>
 	return result;
 }
 
+template<class ST>
+CFeatures* CDenseFeatures<ST>::copy_dimension_subset(SGVector<index_t> dims)
+{
+	SG_DEBUG("Entering!\n");
+
+	// sanity checks
+	index_t max=CMath::max(dims.vector, dims.vlen);
+	index_t min=CMath::min(dims.vector, dims.vlen);
+	REQUIRE(max<num_features && min>=0,
+			"Provided dimensions is in the range [%d, %d] but they "
+			"have to be within [0, %d]! But it \n", min, max, num_features);
+
+	SGMatrix<ST> feature_matrix_copy(dims.vlen, get_num_vectors());
+
+	for (index_t i=0; i<dims.vlen; ++i)
+	{
+		for (index_t j=0; j<get_num_vectors(); ++j)
+		{
+			index_t real_idx=m_subset_stack->subset_idx_conversion(j);
+			feature_matrix_copy(i, j)=feature_matrix(dims[i], real_idx);
+		}
+	}
+
+	CFeatures* result=new CDenseFeatures(feature_matrix_copy);
+	SG_REF(result);
+
+	SG_DEBUG("Leaving!\n");
+	return result;
+}
+
 template<class ST> ST* CDenseFeatures<ST>::compute_feature_vector(int32_t num, int32_t& len,
 		ST* target)
 {
@@ -631,8 +663,8 @@ template<class ST> void CDenseFeatures<ST>::init()
 
 #define GET_FEATURE_TYPE(f_type, sg_type)	\
 template<> EFeatureType CDenseFeatures<sg_type>::get_feature_type() const \
-{ 																			\
-	return f_type; 															\
+{																			\
+	return f_type;															\
 }
 
 GET_FEATURE_TYPE(F_BOOL, bool)
@@ -880,7 +912,7 @@ template<> float64_t CDenseFeatures<float64_t>::dense_dot(
 	float64_t* vec1 = get_feature_vector(vec_idx1, vlen, vfree);
 
 	ASSERT(vlen == num_features)
-	float64_t result = SGVector<float64_t>::dot(vec1, vec2, num_features);
+	float64_t result = CMath::dot(vec1, vec2, num_features);
 
 	free_feature_vector(vec1, vec_idx1, vfree);
 
@@ -995,7 +1027,7 @@ template<class ST> CFeatures* CDenseFeatures<ST>::create_merged_copy(
 	/* count number of vectors (not elements) processed so far */
 	index_t num_processed=num_vectors;
 
-	/* now copy data of other features bock wise */
+	/* now copy data of other features block wise */
 	other=others->get_first_element();
 	while (other)
 	{
@@ -1052,6 +1084,13 @@ void CDenseFeatures<ST>::save(CFile* writer)
 	feature_matrix.save(writer);
 }
 
+template< class ST > CDenseFeatures< ST >* CDenseFeatures< ST >::obtain_from_generic(CFeatures* const base_features)
+{
+	REQUIRE(base_features->get_feature_class() == C_DENSE,
+			"base_features must be of dynamic type CDenseFeatures\n")
+
+	return (CDenseFeatures< ST >*) base_features;
+}
 
 template class CDenseFeatures<bool>;
 template class CDenseFeatures<char>;
