@@ -9,6 +9,9 @@
  */
 #ifndef _FEATURE_TYPES__H__
 #define _FEATURE_TYPES__H__
+
+#include <shogun/lib/config.h>
+
 namespace shogun
 {
 
@@ -49,8 +52,11 @@ namespace shogun
 		C_STREAMING_STRING = 130,
 		C_STREAMING_VW = 140,
 		C_BINNED_DOT = 150,
-        C_DIRECTOR_DOT = 160,
-    C_LATENT = 170,
+		C_DIRECTOR_DOT = 160,
+		C_LATENT = 170,
+		C_MATRIX = 180,
+		C_FACTOR_GRAPH = 190,
+		C_INDEX = 200,
 		C_ANY = 1000
 	};
 
