@@ -13,6 +13,7 @@
 
 #include <shogun/io/SerializableHdf5File.h>
 #include <shogun/io/SerializableHdf5Reader00.h>
+#include <shogun/lib/SGSparseVector.h>
 
 #define NOT_OPEN                   ((hid_t) -1)
 
@@ -105,7 +106,11 @@ CSerializableHdf5File::ptype2hdf5(EPrimitiveType ptype)
 	case PT_FLOAT32: return H5T_NATIVE_FLOAT; break;
 	case PT_FLOAT64: return H5T_NATIVE_DOUBLE; break;
 	case PT_FLOATMAX: return H5T_NATIVE_LDOUBLE; break;
+	case PT_COMPLEX128: return NOT_OPEN; break;
 	case PT_SGOBJECT: return NOT_OPEN; break;
+	case PT_UNDEFINED:
+		SG_SERROR("Type undefined\n");
+		return NOT_OPEN;
 	}
 
 	return NOT_OPEN;
@@ -151,6 +156,7 @@ CSerializableHdf5File::isequal_stype2hdf5(EStructType stype,
 
 	bool to_close = false;
 	switch (stype) {
+	case ST_UNDEFINED:
 	case ST_NONE: break;
 	case ST_STRING:
 		to_close = true; pbuf = H5Tvlen_create(pbuf); break;
