@@ -10,16 +10,21 @@
 #ifndef INDEXBLOCKGROUP_H_
 #define INDEXBLOCKGROUP_H_
 
-#include <shogun/lib/IndexBlock.h>
+#include <shogun/lib/config.h>
+
+#include <shogun/lib/SGVector.h>
 #include <shogun/lib/IndexBlockRelation.h>
 
 namespace shogun
 {
 
-/** @brief class IndexBlockGroup used to represent 
+class CIndexBlock;
+class CList;
+
+/** @brief class IndexBlockGroup used to represent
  * group-based feature relation.
  *
- * Currently can be constructed with a few CIndexBlock 
+ * Currently can be constructed with a few CIndexBlock
  * instances.
  */
 class CIndexBlockGroup : public CIndexBlockRelation
@@ -42,7 +47,7 @@ public:
 	 */
 	void remove_block(CIndexBlock* block);
 
-	/** returns information about IndexBlocks in 
+	/** returns information about IndexBlocks in
 	 * SLEP "ind" format
 	 */
 	SGVector<index_t> get_SLEP_ind();
