@@ -1,3 +1,16 @@
+/*
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * Written (W) 2011-2013 Heiko Strathmann
+ * Written (W) 2012 Fernando Jose Iglesias Garcia
+ * Written (W) 2010,2012 Soeren Sonnenburg
+ * Copyright (C) 2010 Berlin Institute of Technology
+ * Copyright (C) 2012 Soeren Sonnenburg
+ */
+
 #include <shogun/lib/config.h>
 #include <shogun/io/SGIO.h>
 #include <shogun/io/File.h>
@@ -5,7 +18,11 @@
 #include <shogun/lib/SGVector.h>
 #include <shogun/mathematics/Math.h>
 #include <shogun/mathematics/lapack.h>
-#include <shogun/lib/SGMatrixList.h>
+#include <limits>
+
+#ifdef HAVE_EIGEN3
+#include <shogun/mathematics/eigen3.h>
+#endif
 
 namespace shogun {
 
@@ -39,6 +56,22 @@ SGMatrix<T>::SGMatrix(const SGMatrix &orig) : SGReferencedData(orig)
 	copy_data(orig);
 }
 
+#ifdef HAVE_EIGEN3
+template <class T>
+SGMatrix<T>::SGMatrix(EigenMatrixXt& mat)
+: SGReferencedData(false), matrix(mat.data()), 
+	num_rows(mat.rows()), num_cols(mat.cols())
+{
+
+}
+
+template <class T> 
+SGMatrix<T>::operator EigenMatrixXtMap() const
+{
+	return EigenMatrixXtMap(matrix, num_rows, num_cols);
+}
+#endif
+
 template <class T>
 SGMatrix<T>::~SGMatrix()
 {
@@ -63,7 +96,7 @@ bool SGMatrix<T>::equals(SGMatrix<T>& other)
 	if (num_rows!=other.num_rows || num_cols!=other.num_cols)
 		return false;
 
-	for (index_t i=0; i<num_rows*num_cols; ++i)
+	for (int64_t i=0; i<int64_t(num_rows)*num_cols; ++i)
 	{
 		if (matrix[i]!=other.matrix[i])
 			return false;
@@ -75,22 +108,115 @@ bool SGMatrix<T>::equals(SGMatrix<T>& other)
 template <class T>
 void SGMatrix<T>::set_const(T const_elem)
 {
-	for (index_t i=0; i<num_rows*num_cols; i++)
+	for (int64_t i=0; i<int64_t(num_rows)*num_cols; i++)
 		matrix[i]=const_elem ;
 }
 
 template <class T>
 void SGMatrix<T>::zero()
 {
-	if (matrix && (num_rows*num_cols))
+	if (matrix && (int64_t(num_rows)*num_cols))
 		set_const(0);
 }
 
+template <>
+void SGMatrix<complex128_t>::zero()
+{
+	if (matrix && (int64_t(num_rows)*num_cols))
+		set_const(complex128_t(0.0));
+}
+
+template <class T>
+bool SGMatrix<T>::is_symmetric()
+{
+	if (num_rows!=num_cols)
+		return false;
+	for (int i=0; i<num_rows; ++i)
+	{
+		for (int j=i+1; j<num_cols; ++j)
+		{
+			if (matrix[j*num_rows+i]!=matrix[i*num_rows+j])
+				return false;
+		}
+	}
+	return true;
+}
+
+template <>
+bool SGMatrix<float32_t>::is_symmetric()
+{
+	if (num_rows!=num_cols)
+		return false;
+	for (int i=0; i<num_rows; ++i)
+	{
+		for (int j=i+1; j<num_cols; ++j)
+		{
+			if (!CMath::fequals<float32_t>(matrix[j*num_rows+i],
+						matrix[i*num_rows+j], FLT_EPSILON))
+				return false;
+		}
+	}
+	return true;
+}
+
+template <>
+bool SGMatrix<float64_t>::is_symmetric()
+{
+	if (num_rows!=num_cols)
+		return false;
+	for (int i=0; i<num_rows; ++i)
+	{
+		for (int j=i+1; j<num_cols; ++j)
+		{
+			if (!CMath::fequals<float64_t>(matrix[j*num_rows+i],
+						matrix[i*num_rows+j], DBL_EPSILON))
+				return false;
+		}
+	}
+	return true;
+}
+
+template <>
+bool SGMatrix<floatmax_t>::is_symmetric()
+{
+	if (num_rows!=num_cols)
+		return false;
+	for (int i=0; i<num_rows; ++i)
+	{
+		for (int j=i+1; j<num_cols; ++j)
+		{
+			if (!CMath::fequals<floatmax_t>(matrix[j*num_rows+i],
+						matrix[i*num_rows+j], LDBL_EPSILON))
+				return false;
+		}
+	}
+	return true;
+}
+
+template <>
+bool SGMatrix<complex128_t>::is_symmetric()
+{
+	if (num_rows!=num_cols)
+		return false;
+	for (int i=0; i<num_rows; ++i)
+	{
+		for (int j=i+1; j<num_cols; ++j)
+		{
+			if (!(CMath::fequals<float64_t>(matrix[j*num_rows+i].real(),
+						matrix[i*num_rows+j].real(), DBL_EPSILON) &&
+					CMath::fequals<float64_t>(matrix[j*num_rows+i].imag(),
+						matrix[i*num_rows+j].imag(), DBL_EPSILON)))
+				return false;
+		}
+	}
+	return true;
+}
+
 template <class T>
 T SGMatrix<T>::max_single()
 {
 	T max=matrix[0];
-	for (index_t i=1; i<num_rows*num_cols; ++i)
+	for (int64_t i=1; i<int64_t(num_rows)*num_cols; ++i)
 	{
 		if (matrix[i]>max)
 			max=matrix[i];
@@ -99,6 +225,13 @@ T SGMatrix<T>::max_single()
 	return max;
 }
 
+template <>
+complex128_t SGMatrix<complex128_t>::max_single()
+{
+	SG_SERROR("SGMatrix::max_single():: Not supported for complex128_t\n");
+	return complex128_t(0.0);
+}
+
 template <class T>
 SGMatrix<T> SGMatrix<T>::clone()
 {
@@ -121,10 +254,10 @@ void SGMatrix<T>::transpose_matrix(
 	T*& matrix, int32_t& num_feat, int32_t& num_vec)
 {
 	/* this should be done in-place! Heiko */
-	T* transposed=SG_MALLOC(T, num_vec*num_feat);
-	for (int32_t i=0; i<num_vec; i++)
+	T* transposed=SG_MALLOC(T, int64_t(num_vec)*num_feat);
+	for (int64_t i=0; i<num_vec; i++)
 	{
-		for (int32_t j=0; j<num_feat; j++)
+		for (int64_t j=0; j<num_feat; j++)
 			transposed[i+j*num_vec]=matrix[i*num_feat+j];
 	}
 
@@ -137,9 +270,9 @@ void SGMatrix<T>::transpose_matrix(
 template <class T>
 void SGMatrix<T>::create_diagonal_matrix(T* matrix, T* v,int32_t size)
 {
-	for(int32_t i=0;i<size;i++)
+	for(int64_t i=0;i<size;i++)
 	{
-		for(int32_t j=0;j<size;j++)
+		for(int64_t j=0;j<size;j++)
 		{
 			if(i==j)
 				matrix[j*size+i]=v[i];
@@ -154,7 +287,7 @@ float64_t SGMatrix<T>::trace(
 	float64_t* mat, int32_t cols, int32_t rows)
 {
 	float64_t trace=0;
-	for (int32_t i=0; i<rows; i++)
+	for (int64_t i=0; i<rows; i++)
 		trace+=mat[i*cols+i];
 	return trace;
 }
@@ -164,10 +297,10 @@ T* SGMatrix<T>::get_row_sum(T* matrix, int32_t m, int32_t n)
 {
 	T* rowsums=SG_CALLOC(T, n);
 
-	for (int32_t i=0; i<n; i++)
+	for (int64_t i=0; i<n; i++)
 	{
-		for (int32_t j=0; j<m; j++)
-			rowsums[i]+=matrix[j+int64_t(i)*m];
+		for (int64_t j=0; j<m; j++)
+			rowsums[i]+=matrix[j+i*m];
 	}
 	return rowsums;
 }
@@ -177,10 +310,10 @@ T* SGMatrix<T>::get_column_sum(T* matrix, int32_t m, int32_t n)
 {
 	T* colsums=SG_CALLOC(T, m);
 
-	for (int32_t i=0; i<n; i++)
+	for (int64_t i=0; i<n; i++)
 	{
-		for (int32_t j=0; j<m; j++)
-			colsums[j]+=matrix[j+int64_t(i)*m];
+		for (int64_t j=0; j<m; j++)
+			colsums[j]+=matrix[j+i*m];
 	}
 	return colsums;
 }
@@ -206,10 +339,10 @@ void SGMatrix<T>::center_matrix(T* matrix, int32_t m, int32_t n)
 
 	T s=SGVector<T>::sum(rowsums, n)/num_data;
 
-	for (int32_t i=0; i<n; i++)
+	for (int64_t i=0; i<n; i++)
 	{
-		for (int32_t j=0; j<m; j++)
-			matrix[int64_t(i)*m+j]+=s-colsums[j]-rowsums[i];
+		for (int64_t j=0; j<m; j++)
+			matrix[i*m+j]+=s-colsums[j]-rowsums[i];
 	}
 
 	SG_FREE(rowsums);
@@ -224,10 +357,10 @@ void SGMatrix<T>::remove_column_mean()
 	T* means=get_row_sum(matrix, num_rows, num_cols);
 
 	/* substract column mean from every corresponding entry */
-	for (index_t i=0; i<num_cols; ++i)
+	for (int64_t i=0; i<num_cols; ++i)
 	{
 		means[i]/=num_rows;
-		for (index_t j=0; j<num_rows; ++j)
+		for (int64_t j=0; j<num_rows; ++j)
 			matrix[i*num_rows+j]-=means[i];
 	}
 
@@ -254,10 +387,10 @@ void SGMatrix<bool>::display_matrix(
 {
 	ASSERT(rows>=0 && cols>=0)
 	SG_SPRINT("%s%s=[\n", prefix, name)
-	for (int32_t i=0; i<rows; i++)
+	for (int64_t i=0; i<rows; i++)
 	{
 		SG_SPRINT("%s[", prefix)
-		for (int32_t j=0; j<cols; j++)
+		for (int64_t j=0; j<cols; j++)
 			SG_SPRINT("%s\t%d%s", prefix, matrix[j*rows+i] ? 1 : 0,
 				j==cols-1? "" : ",");
 		SG_SPRINT("%s]%s\n", prefix, i==rows-1? "" : ",")
@@ -272,10 +405,10 @@ void SGMatrix<char>::display_matrix(
 {
 	ASSERT(rows>=0 && cols>=0)
 	SG_SPRINT("%s%s=[\n", prefix, name)
-	for (int32_t i=0; i<rows; i++)
+	for (int64_t i=0; i<rows; i++)
 	{
 		SG_SPRINT("%s[", prefix)
-		for (int32_t j=0; j<cols; j++)
+		for (int64_t j=0; j<cols; j++)
 			SG_SPRINT("%s\t%c%s", prefix, matrix[j*rows+i],
 				j==cols-1? "" : ",");
 		SG_SPRINT("%s]%s\n", prefix, i==rows-1? "" : ",")
@@ -290,10 +423,10 @@ void SGMatrix<int8_t>::display_matrix(
 {
 	ASSERT(rows>=0 && cols>=0)
 	SG_SPRINT("%s%s=[\n", prefix, name)
-	for (int32_t i=0; i<rows; i++)
+	for (int64_t i=0; i<rows; i++)
 	{
 		SG_SPRINT("%s[", prefix)
-		for (int32_t j=0; j<cols; j++)
+		for (int64_t j=0; j<cols; j++)
 			SG_SPRINT("%s\t%d%s", prefix, matrix[j*rows+i],
 				j==cols-1? "" : ",");
 		SG_SPRINT("%s]%s\n", prefix, i==rows-1? "" : ",")
@@ -308,10 +441,10 @@ void SGMatrix<uint8_t>::display_matrix(
 {
 	ASSERT(rows>=0 && cols>=0)
 	SG_SPRINT("%s%s=[\n", prefix, name)
-	for (int32_t i=0; i<rows; i++)
+	for (int64_t i=0; i<rows; i++)
 	{
 		SG_SPRINT("%s[", prefix)
-		for (int32_t j=0; j<cols; j++)
+		for (int64_t j=0; j<cols; j++)
 			SG_SPRINT("%s\t%d%s", prefix, matrix[j*rows+i],
 				j==cols-1? "" : ",");
 		SG_SPRINT("%s]%s\n", prefix, i==rows-1? "" : ",")
@@ -326,10 +459,10 @@ void SGMatrix<int16_t>::display_matrix(
 {
 	ASSERT(rows>=0 && cols>=0)
 	SG_SPRINT("%s%s=[\n", prefix, name)
-	for (int32_t i=0; i<rows; i++)
+	for (int64_t i=0; i<rows; i++)
 	{
 		SG_SPRINT("%s[", prefix)
-		for (int32_t j=0; j<cols; j++)
+		for (int64_t j=0; j<cols; j++)
 			SG_SPRINT("%s\t%d%s", prefix, matrix[j*rows+i],
 				j==cols-1? "" : ",");
 		SG_SPRINT("%s]%s\n", prefix, i==rows-1? "" : ",")
@@ -344,10 +477,10 @@ void SGMatrix<uint16_t>::display_matrix(
 {
 	ASSERT(rows>=0 && cols>=0)
 	SG_SPRINT("%s%s=[\n", prefix, name)
-	for (int32_t i=0; i<rows; i++)
+	for (int64_t i=0; i<rows; i++)
 	{
 		SG_SPRINT("%s[", prefix)
-		for (int32_t j=0; j<cols; j++)
+		for (int64_t j=0; j<cols; j++)
 			SG_SPRINT("%s\t%d%s", prefix, matrix[j*rows+i],
 				j==cols-1? "" : ",");
 		SG_SPRINT("%s]%s\n", prefix, i==rows-1? "" : ",")
@@ -363,10 +496,10 @@ void SGMatrix<int32_t>::display_matrix(
 {
 	ASSERT(rows>=0 && cols>=0)
 	SG_SPRINT("%s%s=[\n", prefix, name)
-	for (int32_t i=0; i<rows; i++)
+	for (int64_t i=0; i<rows; i++)
 	{
 		SG_SPRINT("%s[", prefix)
-		for (int32_t j=0; j<cols; j++)
+		for (int64_t j=0; j<cols; j++)
 			SG_SPRINT("%s\t%d%s", prefix, matrix[j*rows+i],
 				j==cols-1? "" : ",");
 		SG_SPRINT("%s]%s\n", prefix, i==rows-1? "" : ",")
@@ -381,10 +514,10 @@ void SGMatrix<uint32_t>::display_matrix(
 {
 	ASSERT(rows>=0 && cols>=0)
 	SG_SPRINT("%s%s=[\n", prefix, name)
-	for (int32_t i=0; i<rows; i++)
+	for (int64_t i=0; i<rows; i++)
 	{
 		SG_SPRINT("%s[", prefix)
-		for (int32_t j=0; j<cols; j++)
+		for (int64_t j=0; j<cols; j++)
 			SG_SPRINT("%s\t%d%s", prefix, matrix[j*rows+i],
 				j==cols-1? "" : ",");
 		SG_SPRINT("%s]%s\n", prefix, i==rows-1? "" : ",")
@@ -398,10 +531,10 @@ void SGMatrix<int64_t>::display_matrix(
 {
 	ASSERT(rows>=0 && cols>=0)
 	SG_SPRINT("%s%s=[\n", prefix, name)
-	for (int32_t i=0; i<rows; i++)
+	for (int64_t i=0; i<rows; i++)
 	{
 		SG_SPRINT("%s[", prefix)
-		for (int32_t j=0; j<cols; j++)
+		for (int64_t j=0; j<cols; j++)
 			SG_SPRINT("%s\t%d%s", prefix, matrix[j*rows+i],
 				j==cols-1? "" : ",");
 		SG_SPRINT("%s]%s\n", prefix, i==rows-1? "" : ",")
@@ -416,10 +549,10 @@ void SGMatrix<uint64_t>::display_matrix(
 {
 	ASSERT(rows>=0 && cols>=0)
 	SG_SPRINT("%s%s=[\n", prefix, name)
-	for (int32_t i=0; i<rows; i++)
+	for (int64_t i=0; i<rows; i++)
 	{
 		SG_SPRINT("%s[", prefix)
-		for (int32_t j=0; j<cols; j++)
+		for (int64_t j=0; j<cols; j++)
 			SG_SPRINT("%s\t%d%s", prefix, matrix[j*rows+i],
 				j==cols-1? "" : ",");
 		SG_SPRINT("%s]%s\n", prefix, i==rows-1? "" : ",")
@@ -434,10 +567,10 @@ void SGMatrix<float32_t>::display_matrix(
 {
 	ASSERT(rows>=0 && cols>=0)
 	SG_SPRINT("%s%s=[\n", prefix, name)
-	for (int32_t i=0; i<rows; i++)
+	for (int64_t i=0; i<rows; i++)
 	{
 		SG_SPRINT("%s[", prefix)
-		for (int32_t j=0; j<cols; j++)
+		for (int64_t j=0; j<cols; j++)
 			SG_SPRINT("%s\t%.18g%s", prefix, (float) matrix[j*rows+i],
 				j==cols-1? "" : ",");
 		SG_SPRINT("%s]%s\n", prefix, i==rows-1? "" : ",")
@@ -452,10 +585,10 @@ void SGMatrix<float64_t>::display_matrix(
 {
 	ASSERT(rows>=0 && cols>=0)
 	SG_SPRINT("%s%s=[\n", prefix, name)
-	for (int32_t i=0; i<rows; i++)
+	for (int64_t i=0; i<rows; i++)
 	{
 		SG_SPRINT("%s[", prefix)
-		for (int32_t j=0; j<cols; j++)
+		for (int64_t j=0; j<cols; j++)
 			SG_SPRINT("%s\t%.18g%s", prefix, (double) matrix[j*rows+i],
 				j==cols-1? "" : ",");
 		SG_SPRINT("%s]%s\n", prefix, i==rows-1? "" : ",")
@@ -470,10 +603,10 @@ void SGMatrix<floatmax_t>::display_matrix(
 {
 	ASSERT(rows>=0 && cols>=0)
 	SG_SPRINT("%s%s=[\n", prefix, name)
-	for (int32_t i=0; i<rows; i++)
+	for (int64_t i=0; i<rows; i++)
 	{
 		SG_SPRINT("%s[", prefix)
-		for (int32_t j=0; j<cols; j++)
+		for (int64_t j=0; j<cols; j++)
 			SG_SPRINT("%s\t%.18g%s", prefix, (double) matrix[j*rows+i],
 				j==cols-1? "" : ",");
 		SG_SPRINT("%s]%s\n", prefix, i==rows-1? "" : ",")
@@ -481,6 +614,24 @@ void SGMatrix<floatmax_t>::display_matrix(
 	SG_SPRINT("%s]\n", prefix)
 }
 
+template <>
+void SGMatrix<complex128_t>::display_matrix(
+	const complex128_t* matrix, int32_t rows, int32_t cols, const char* name,
+	const char* prefix)
+{
+	ASSERT(rows>=0 && cols>=0)
+	SG_SPRINT("%s%s=[\n", prefix, name)
+	for (int64_t i=0; i<rows; i++)
+	{
+		SG_SPRINT("%s[", prefix)
+		for (int64_t j=0; j<cols; j++)
+			SG_SPRINT("%s\t(%.18g+i%.18g)%s", prefix, matrix[j*rows+i].real(),
+				matrix[j*rows+i].imag(), j==cols-1? "" : ",");
+		SG_SPRINT("%s]%s\n", prefix, i==rows-1? "" : ",")
+	}
+	SG_SPRINT("%s]\n", prefix)
+}
+
 template <>
 SGMatrix<char> SGMatrix<char>::create_identity_matrix(index_t size, char scale)
 {
@@ -644,20 +795,17 @@ SGMatrix<floatmax_t> SGMatrix<floatmax_t>::create_identity_matrix(index_t size,
 	return I;
 }
 
-
-template <class T>
-SGMatrix<float64_t> SGMatrix<T>::create_centering_matrix(index_t size)
+template <>
+SGMatrix<complex128_t> SGMatrix<complex128_t>::create_identity_matrix(index_t size, complex128_t scale)
 {
-	SGMatrix<float64_t> H=SGMatrix<float64_t>::create_identity_matrix(size, 1.0);
-
-	float64_t subtract=1.0/size;
+	SGMatrix<complex128_t> I(size, size);
 	for (index_t i=0; i<size; ++i)
 	{
-		for (index_t j=0; j<0; ++j)
-			H(i,j)-=subtract;
+		for (index_t j=0; j<size; ++j)
+			I(i,j)=i==j ? scale : complex128_t(0.0);
 	}
 
-	return H;
+	return I;
 }
 
 //Howto construct the pseudo inverse (from "The Matrix Cookbook")
@@ -693,9 +841,9 @@ float64_t* SGMatrix<T>::pinv(
 	wrap_dgesvd(jobu, jobvt, m, n, matrix, lda, s, u, ldu, vt, ldvt, &info);
 	ASSERT(info==0)
 
-	for (int32_t i=0; i<n; i++)
+	for (int64_t i=0; i<n; i++)
 	{
-		for (int32_t j=0; j<lsize; j++)
+		for (int64_t j=0; j<lsize; j++)
 			vt[i*n+j]=vt[i*n+j]/s[j];
 	}
 
@@ -722,7 +870,7 @@ void SGMatrix<T>::inverse(SGMatrix<float64_t> matrix)
 template <class T>
 SGVector<float64_t> SGMatrix<T>::compute_eigenvectors(SGMatrix<float64_t> matrix)
 {
-	if (matrix.num_rows!=matrix.num_rows)
+	if (matrix.num_rows!=matrix.num_cols)
 	{
 		SG_SERROR("SGMatrix::compute_eigenvectors(SGMatrix<float64_t>): matrix"
 				" rows and columns are not equal!\n");
@@ -801,12 +949,19 @@ SGMatrix<float64_t> SGMatrix<T>::matrix_multiply(
 			A.matrix, A.num_rows, B.matrix, B.num_rows,
 			0.0, C.matrix, C.num_rows);
 #else
+	/* C(i,j) = scale * \Sigma A(i,k)*B(k,j) */
 	for (int32_t i=0; i<rows_A; i++)
 	{
 		for (int32_t j=0; j<cols_B; j++)
 		{
 			for (int32_t k=0; k<cols_A; k++)
-				C(i,j) += A(i,k)*B(k,j);
+			{
+				float64_t x1=transpose_A ? A(k,i):A(i,k);
+				float64_t x2=transpose_B ? B(j,k):B(k,j);
+				C(i,j)+=x1*x2;
+			}
+
+			C(i,j)*=scale;
 		}
 	}
 #endif //HAVE_LAPACK
@@ -876,10 +1031,20 @@ void SGMatrix<T>::load(CFile* loader)
 	unref();
 
 	SG_SET_LOCALE_C;
-	loader->get_matrix(matrix, num_rows, num_cols);
+	SGMatrix<T> mat;
+	loader->get_matrix(mat.matrix, mat.num_rows, mat.num_cols);
+	copy_data(mat);
+	copy_refcount(mat);
+	ref();
 	SG_RESET_LOCALE;
 }
 
+template<>
+void SGMatrix<complex128_t>::load(CFile* loader)
+{
+	SG_SERROR("SGMatrix::load():: Not supported for complex128_t\n");
+}
+
 template<class T>
 void SGMatrix<T>::save(CFile* writer)
 {
@@ -889,6 +1054,37 @@ void SGMatrix<T>::save(CFile* writer)
 	SG_RESET_LOCALE;
 }
 
+template<>
+void SGMatrix<complex128_t>::save(CFile* saver)
+{
+	SG_SERROR("SGMatrix::save():: Not supported for complex128_t\n");
+}
+
+template<class T>
+SGVector<T> SGMatrix<T>::get_row_vector(index_t row) const
+{
+	SGVector<T> rowv(num_cols);
+	for (int64_t i = 0; i < num_cols; i++)
+	{
+		rowv[i] = matrix[i*num_rows+row];
+	}
+	return rowv;
+}
+
+template<class T>
+SGVector<T> SGMatrix<T>::get_diagonal_vector() const
+{
+	index_t diag_vlen=CMath::min(num_cols, num_rows);
+	SGVector<T> diag(diag_vlen);
+
+	for (int64_t i=0; i<diag_vlen; i++)
+	{
+		diag[i]=matrix[i*num_rows+i];
+	}
+
+	return diag;
+}
+
 template class SGMatrix<bool>;
 template class SGMatrix<char>;
 template class SGMatrix<int8_t>;
@@ -902,4 +1098,5 @@ template class SGMatrix<uint64_t>;
 template class SGMatrix<float32_t>;
 template class SGMatrix<float64_t>;
 template class SGMatrix<floatmax_t>;
+template class SGMatrix<complex128_t>;
 }
