@@ -4,13 +4,18 @@
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
+ * Written (W) 2013 Thoralf Klein
+ * Written (W) 2011-2013 Heiko Strathmann
+ * Written (W) 2013 Soumyajit De
  * Written (W) 2012 Fernando José Iglesias García
  * Written (W) 2010,2012 Soeren Sonnenburg
  * Copyright (C) 2010 Berlin Institute of Technology
  * Copyright (C) 2012 Soeren Sonnenburg
  */
+
 #include <shogun/lib/config.h>
 #include <shogun/lib/SGVector.h>
+#include <shogun/lib/SGMatrix.h>
 #include <shogun/lib/SGSparseVector.h>
 #include <shogun/lib/SGReferencedData.h>
 #include <shogun/io/File.h>
@@ -21,6 +26,46 @@
 
 #include <shogun/mathematics/eigen3.h>
 
+#define COMPLEX128_ERROR_NOARG(function) \
+template <> \
+void SGVector<complex128_t>::function() \
+{ \
+	SG_SERROR("SGVector::%s():: Not supported for complex128_t\n",\
+		#function);\
+}
+
+#define BOOL_ERROR_ONEARG(function) \
+template <> \
+void SGVector<bool>::function(bool a) \
+{ \
+	SG_SERROR("SGVector::%s():: Not supported for bool\n",\
+		#function);\
+}
+
+#define COMPLEX128_ERROR_ONEARG(function) \
+template <> \
+void SGVector<complex128_t>::function(complex128_t a) \
+{ \
+	SG_SERROR("SGVector::%s():: Not supported for complex128_t\n",\
+		#function);\
+}
+
+#define COMPLEX128_ERROR_TWOARGS(function) \
+template <> \
+void SGVector<complex128_t>::function(complex128_t a, complex128_t b) \
+{ \
+	SG_SERROR("SGVector::%s():: Not supported for complex128_t\n",\
+		#function);\
+}
+
+#define COMPLEX128_ERROR_THREEARGS(function) \
+template <> \
+void SGVector<complex128_t>::function(complex128_t a, complex128_t b,\
+	complex128_t c) \
+{ \
+	SG_SERROR("SGVector::%s():: Not supported for complex128_t\n",\
+		#function);\
+}
 
 namespace shogun {
 
@@ -49,12 +94,46 @@ SGVector<T>::SGVector(const SGVector &orig) : SGReferencedData(orig)
 	copy_data(orig);
 }
 
+template<class T>
+void SGVector<T>::set(SGVector<T> orig)
+{
+	*this = SGVector<T>(orig);
+}
+
 template<class T>
 SGVector<T>::~SGVector()
 {
 	unref();
 }
 
+#ifdef HAVE_EIGEN3
+template <class T> 
+SGVector<T>::SGVector(EigenVectorXt& vec)
+: SGReferencedData(false), vector(vec.data()), vlen(vec.size())
+{
+	
+}
+
+template <class T> 
+SGVector<T>::SGVector(EigenRowVectorXt& vec)
+: SGReferencedData(false), vector(vec.data()), vlen(vec.size())
+{
+	
+}
+
+template <class T> 
+SGVector<T>::operator EigenVectorXtMap() const
+{
+	return EigenVectorXtMap(vector, vlen);
+}
+
+template <class T> 
+SGVector<T>::operator EigenRowVectorXtMap() const
+{
+	return EigenRowVectorXtMap(vector, vlen);
+}
+#endif
+
 template<class T>
 void SGVector<T>::zero()
 {
@@ -62,6 +141,13 @@ void SGVector<T>::zero()
 		set_const(0);
 }
 
+template <>
+void SGVector<complex128_t>::zero()
+{
+	if (vector && vlen)
+		set_const(complex128_t(0.0));
+}
+
 template<class T>
 void SGVector<T>::set_const(T const_elem)
 {
@@ -89,54 +175,15 @@ void SGVector<T>::range_fill(T start)
 	range_fill_vector(vector, vlen, start);
 }
 
+COMPLEX128_ERROR_ONEARG(range_fill)
+
 template<class T>
 void SGVector<T>::random(T min_value, T max_value)
 {
 	random_vector(vector, vlen, min_value, max_value);
 }
 
-template<class T>
-void SGVector<T>::randperm()
-{
-	randperm(vector, vlen);
-}
-
-template <class T>
-void SGVector<T>::qsort()
-{
-	CMath::qsort<T>(vector, vlen);
-}
-
-/** Helper functor for the function argsort */
-template<class T>
-struct IndexSorter
-{
-	/** constructor */
-	IndexSorter(const SGVector<T> *vec) { data = vec->vector; }
-
-	/** access operator */
-	bool operator() (index_t i, index_t j) const
-	{
-		return data[i] < data[j];
-	}
-
-	/** data */
-	const T* data;
-};
-
-template<class T>
-SGVector<index_t> SGVector<T>::argsort()
-{
-	IndexSorter<T> cmp(this);
-	SGVector<index_t> idx(vlen);
-	for (index_t i=0; i < vlen; ++i)
-		idx[i] = i;
-
-	std::sort(idx.vector, idx.vector+vlen, cmp);
-
-	return idx;
-}
-
+COMPLEX128_ERROR_TWOARGS(random)
 
 template <class T>
 index_t SGVector<T>::find_position_to_insert(T element)
@@ -150,6 +197,14 @@ index_t SGVector<T>::find_position_to_insert(T element)
 	return i;
 }
 
+template <>
+index_t SGVector<complex128_t>::find_position_to_insert(complex128_t element)
+{
+	SG_SERROR("SGVector::find_position_to_insert():: \
+		Not supported for complex128_t\n");
+	return index_t(-1);
+}
+
 template<class T>
 SGVector<T> SGVector<T>::clone() const
 {
@@ -178,6 +233,13 @@ void SGVector<T>::range_fill_vector(T* vec, int32_t len, T start)
 		vec[i]=i+start;
 }
 
+template <>
+void SGVector<complex128_t>::range_fill_vector(complex128_t* vec,
+	int32_t len, complex128_t start)
+{
+	SG_SERROR("SGVector::range_fill_vector():: \
+		Not supported for complex128_t\n");
+}
 
 template<class T>
 const T& SGVector<T>::get_element(index_t index)
@@ -276,6 +338,21 @@ void SGVector<T>::free_data()
 	vlen=0;
 }
 
+template<class T>
+bool SGVector<T>::equals(SGVector<T>& other)
+{
+	if (other.vlen!=vlen)
+		return false;
+
+	for (index_t i=0; i<vlen; ++i)
+	{
+		if (other.vector[i]!=vector[i])
+			return false;
+	}
+
+	return true;
+}
+
 template<class T>
 void SGVector<T>::display_vector(const char* name,
 		const char* prefix) const
@@ -319,7 +396,7 @@ void SGVector<uint8_t>::display_vector(const uint8_t* vector, int32_t n, const c
 	ASSERT(n>=0)
 	SG_SPRINT("%s%s=[", prefix, name)
 	for (int32_t i=0; i<n; i++)
-		SG_SPRINT("%s%d%s", prefix, vector[i], i==n-1? "" : ",")
+		SG_SPRINT("%s%u%s", prefix, vector[i], i==n-1? "" : ",")
 	SG_SPRINT("%s]\n", prefix)
 }
 
@@ -341,7 +418,7 @@ void SGVector<uint16_t>::display_vector(const uint16_t* vector, int32_t n, const
 	ASSERT(n>=0)
 	SG_SPRINT("%s%s=[", prefix, name)
 	for (int32_t i=0; i<n; i++)
-		SG_SPRINT("%s%d%s", prefix, vector[i], i==n-1? "" : ",")
+		SG_SPRINT("%s%u%s", prefix, vector[i], i==n-1? "" : ",")
 	SG_SPRINT("%s]\n", prefix)
 }
 
@@ -374,7 +451,7 @@ void SGVector<uint32_t>::display_vector(const uint32_t* vector, int32_t n, const
 	ASSERT(n>=0)
 	SG_SPRINT("%s%s=[", prefix, name)
 	for (int32_t i=0; i<n; i++)
-		SG_SPRINT("%s%d%s", prefix, vector[i], i==n-1? "" : ",")
+		SG_SPRINT("%s%u%s", prefix, vector[i], i==n-1? "" : ",")
 	SG_SPRINT("%s]\n", prefix)
 }
 
@@ -437,6 +514,20 @@ void SGVector<floatmax_t>::display_vector(const floatmax_t* vector, int32_t n,
 	SG_SPRINT("%s]\n", prefix)
 }
 
+template <>
+void SGVector<complex128_t>::display_vector(const complex128_t* vector, int32_t n,
+		const char* name, const char* prefix)
+{
+	ASSERT(n>=0)
+	SG_SPRINT("%s%s=[", prefix, name)
+	for (int32_t i=0; i<n; i++)
+	{
+		SG_SPRINT("%s(%.36lg+i%.36lg)%s", prefix, vector[i].real(),
+				vector[i].imag(), i==n-1? "" : ",");
+	}
+	SG_SPRINT("%s]\n", prefix)
+}
+
 template <class T>
 void SGVector<T>::vec1_plus_scalar_times_vec2(T* vec1,
 		const T scalar, const T* vec2, int32_t n)
@@ -471,42 +562,6 @@ void SGVector<float32_t>::vec1_plus_scalar_times_vec2(float32_t* vec1,
 #endif
 }
 
-template <class T>
-float64_t SGVector<T>::dot(const float64_t* v1, const float64_t* v2, int32_t n)
-{
-	float64_t r=0;
-#ifdef HAVE_EIGEN3
-	Eigen::Map<const Eigen::VectorXd> ev1(v1,n);
-	Eigen::Map<const Eigen::VectorXd> ev2(v2,n);
-	r = ev1.dot(ev2);
-#elif HAVE_LAPACK
-	int32_t skip=1;
-	r = cblas_ddot(n, v1, skip, v2, skip);
-#else
-	for (int32_t i=0; i<n; i++)
-		r+=v1[i]*v2[i];
-#endif
-	return r;
-}
-
-template <class T>
-float32_t SGVector<T>::dot(const float32_t* v1, const float32_t* v2, int32_t n)
-{
-	float32_t r=0;
-#ifdef HAVE_EIGEN3
-	Eigen::Map<const Eigen::VectorXf> ev1(v1,n);
-	Eigen::Map<const Eigen::VectorXf> ev2(v2,n);
-	r = ev1.dot(ev2);
-#elif HAVE_LAPACK
-	int32_t skip=1;
-	r = cblas_sdot(n, v1, skip, v2, skip);
-#else
-	for (int32_t i=0; i<n; i++)
-		r+=v1[i]*v2[i];
-#endif
-	return r;
-}
-
 template <class T>
 	void SGVector<T>::random_vector(T* vec, int32_t len, T min_value, T max_value)
 	{
@@ -514,61 +569,11 @@ template <class T>
 			vec[i]=CMath::random(min_value, max_value);
 	}
 
-template <class T>
-SGVector<T> SGVector<T>::randperm_vec(int32_t n)
+template <>
+void SGVector<complex128_t>::random_vector(complex128_t* vec, int32_t len,
+	complex128_t min_value, complex128_t max_value)
 {
-	return SGVector<T>(randperm(n), n);
-}
-
-template <class T>
-T* SGVector<T>::randperm(int32_t n)
-{
-	T* perm = SG_MALLOC(T, n);
-	randperm(perm, n);
-
-	return perm;
-}
-
-template <class T>
-void SGVector<T>::randperm(T* perm, int32_t n)
-{
-	for (int32_t i = 0; i < n; i++)
-		perm[i] = i;
-	permute(perm,n);
-}
-
-
-/** permute */
-template <class T>
-void SGVector<T>::permute(T* vec, int32_t n)
-{
-	for (int32_t i = 0; i < n; i++)
-		CMath::swap(vec[i], vec[CMath::random(i, n-1)]);
-}
-
-template<class T>
-void SGVector<T>::permute()
-{
-	SGVector<T>::permute(vector, vlen);
-}
-
-template <class T>
-void SGVector<T>::permute_vector(SGVector<T> vec)
-{
-	for (index_t i=0; i<vec.vlen; ++i)
-	{
-		CMath::swap(vec.vector[i],
-				vec.vector[CMath::random(i, vec.vlen-1)]);
-	}
-}
-
-template <class T>
-void SGVector<T>::resize(T* &data, int64_t old_size, int64_t new_size)
-{
-	if (old_size==new_size)
-		return;
-
-	data = SG_REALLOC(T, data, old_size, new_size);
+	SG_SNOTIMPLEMENTED
 }
 
 template <>
@@ -682,7 +687,7 @@ float64_t SGVector<float64_t>::twonorm(const float64_t* v, int32_t n)
 #ifdef HAVE_LAPACK
 	norm = cblas_dnrm2(n, v, 1);
 #else
-	norm = CMath::sqrt(SGVector::dot(v, v, n));
+	norm = CMath::sqrt(CMath::dot(v, v, n));
 #endif
 	return norm;
 }
@@ -697,6 +702,15 @@ floatmax_t SGVector<floatmax_t>::twonorm(const floatmax_t* x, int32_t len)
 	return CMath::sqrt(result);
 }
 
+template <>
+complex128_t SGVector<complex128_t>::twonorm(const complex128_t* x, int32_t len)
+{
+	complex128_t result(0.0);
+	for (int32_t i=0; i<len; i++)
+		result+=x[i]*x[i];
+
+	return CMath::sqrt(result);
+}
 
 template <class T>
 float64_t SGVector<T>::onenorm(T* x, int32_t len)
@@ -719,6 +733,13 @@ T SGVector<T>::qsq(T* x, int32_t len, float64_t q)
 	return result;
 }
 
+template <>
+complex128_t SGVector<complex128_t>::qsq(complex128_t* x, int32_t len, float64_t q)
+{
+	SG_SNOTIMPLEMENTED
+	return complex128_t(0.0);
+}
+
 /// || x ||_q
 template <class T>
 T SGVector<T>::qnorm(T* x, int32_t len, float64_t q)
@@ -727,152 +748,11 @@ T SGVector<T>::qnorm(T* x, int32_t len, float64_t q)
 	return CMath::pow((float64_t) qsq(x, len, q), 1.0/q);
 }
 
-/** @return min(vec) */
-template <class T>
-	T SGVector<T>::min(T* vec, int32_t len)
-	{
-		ASSERT(len>0)
-		T minv=vec[0];
-
-		for (int32_t i=1; i<len; i++)
-			minv=CMath::min(vec[i], minv);
-
-		return minv;
-	}
-
-#ifdef HAVE_LAPACK
 template <>
-float64_t SGVector<float64_t>::max_abs(float64_t* vec, int32_t len)
+complex128_t SGVector<complex128_t>::qnorm(complex128_t* x, int32_t len, float64_t q)
 {
-	ASSERT(len>0)
-	int32_t skip = 1;
-	int32_t idx = cblas_idamax(len, vec, skip);
-
-	return CMath::abs(vec[idx]);
-}
-
-template <>
-float32_t SGVector<float32_t>::max_abs(float32_t* vec, int32_t len)
-{
-	ASSERT(len>0)
-	int32_t skip = 1;
-	int32_t idx = cblas_isamax(len, vec, skip);
-
-	return CMath::abs(vec[idx]);
-}
-#endif
-
-/** @return max_abs(vec) */
-template <class T>
-T SGVector<T>::max_abs(T* vec, int32_t len)
-{
-	ASSERT(len>0)
-	T maxv=CMath::abs(vec[0]);
-
-	for (int32_t i=1; i<len; i++)
-		maxv=CMath::max(CMath::abs(vec[i]), maxv);
-
-	return maxv;
-}
-
-/** @return max(vec) */
-template <class T>
-T SGVector<T>::max(T* vec, int32_t len)
-{
-	ASSERT(len>0)
-	T maxv=vec[0];
-
-	for (int32_t i=1; i<len; i++)
-		maxv=CMath::max(vec[i], maxv);
-
-	return maxv;
-}
-
-#ifdef HAVE_LAPACK
-template <>
-int32_t SGVector<float64_t>::arg_max_abs(float64_t* vec, int32_t inc, int32_t len, float64_t* maxv_ptr)
-{
-	ASSERT(len>0 || inc > 0)
-	int32_t idx = cblas_idamax(len, vec, inc);
-
-	if (maxv_ptr != NULL)
-		*maxv_ptr = CMath::abs(vec[idx*inc]);
-
-	return idx;
-}
-
-template <>
-int32_t SGVector<float32_t>::arg_max_abs(float32_t* vec, int32_t inc, int32_t len, float32_t* maxv_ptr)
-{
-	ASSERT(len>0 || inc > 0)
-	int32_t idx = cblas_isamax(len, vec, inc);
-
-	if (maxv_ptr != NULL)
-		*maxv_ptr = CMath::abs(vec[idx*inc]);
-
-	return idx;
-}
-#endif
-
-template <class T>
-int32_t SGVector<T>::arg_max_abs(T * vec, int32_t inc, int32_t len, T * maxv_ptr)
-{
-	ASSERT(len > 0 || inc > 0)
-
-	T maxv = CMath::abs(vec[0]);
-	int32_t maxIdx = 0;
-
-	for (int32_t i = 1, j = inc ; i < len ; i++, j += inc)
-	{
-		if (CMath::abs(vec[j]) > maxv)
-			maxv = CMath::abs(vec[j]), maxIdx = i;
-	}
-
-	if (maxv_ptr != NULL)
-		*maxv_ptr = maxv;
-
-	return maxIdx;
-}
-
-template <class T>
-int32_t SGVector<T>::arg_max(T * vec, int32_t inc, int32_t len, T * maxv_ptr)
-{
-	ASSERT(len > 0 || inc > 0)
-
-	T maxv = vec[0];
-	int32_t maxIdx = 0;
-
-	for (int32_t i = 1, j = inc ; i < len ; i++, j += inc)
-	{
-		if (vec[j] > maxv)
-			maxv = vec[j], maxIdx = i;
-	}
-
-	if (maxv_ptr != NULL)
-		*maxv_ptr = maxv;
-
-	return maxIdx;
-}
-
-/// return arg_min(vec)
-template <class T>
-int32_t SGVector<T>::arg_min(T * vec, int32_t inc, int32_t len, T * minv_ptr)
-{
-	ASSERT(len > 0 || inc > 0)
-
-	T minv = vec[0];
-	int32_t minIdx = 0;
-
-	for (int32_t i = 1, j = inc ; i < len ; i++, j += inc)
-	{
-		if (vec[j] < minv)
-			minv = vec[j], minIdx = i;
-	}
-
-	if (minv_ptr != NULL)
-		*minv_ptr = minv;
-
-	return minIdx;
+	SG_SNOTIMPLEMENTED
+	return complex128_t(0.0);
 }
 
 /// return sum(abs(vec))
@@ -904,13 +784,6 @@ float32_t SGVector<float32_t>::sum_abs(float32_t* vec, int32_t len)
 }
 #endif
 
-/// return sum(abs(vec))
-template <class T>
-bool SGVector<T>::fequal(T x, T y, float64_t precision)
-{
-	return CMath::abs(x-y)<precision;
-}
-
 template <class T>
 int32_t SGVector<T>::unique(T* output, int32_t size)
 {
@@ -925,6 +798,14 @@ int32_t SGVector<T>::unique(T* output, int32_t size)
 	return j;
 }
 
+template <>
+int32_t SGVector<complex128_t>::unique(complex128_t* output, int32_t size)
+{
+	int32_t j=0;
+	SG_SERROR("SGVector::unique():: Not supported for complex128_t\n");
+	return j;
+}
+
 template <class T>
 SGVector<index_t> SGVector<T>::find(T elem)
 {
@@ -965,26 +846,26 @@ void SGVector<T>::scale(T alpha)
 	scale_vector(alpha, vector, vlen);
 }
 
-template<class T> float64_t SGVector<T>::mean() const
-{
-	float64_t cum = 0;
-
-	for ( index_t i = 0 ; i < vlen ; ++i )
-		cum += vector[i];
-
-	return cum/vlen;
-}
-
 template<class T> void SGVector<T>::load(CFile* loader)
 {
 	ASSERT(loader)
 	unref();
 
 	SG_SET_LOCALE_C;
-	loader->get_vector(vector, vlen);
+	SGVector<T> vec;
+	loader->get_vector(vec.vector, vec.vlen);
+	copy_data(vec);
+	copy_refcount(vec);
+	ref();
 	SG_RESET_LOCALE;
 }
 
+template<>
+void SGVector<complex128_t>::load(CFile* loader)
+{
+	SG_SERROR("SGVector::load():: Not supported for complex128_t\n");
+}
+
 template<class T> void SGVector<T>::save(CFile* saver)
 {
 	ASSERT(saver)
@@ -993,42 +874,106 @@ template<class T> void SGVector<T>::save(CFile* saver)
 	saver->set_vector(vector, vlen);
 	SG_RESET_LOCALE;
 }
-		
-#define MATHOP(op)								\
-template <class T> void SGVector<T>::op()		\
-{												\
-	for (int32_t i=0; i<vlen; i++)				\
-		vector[i]=(T) CMath::op((double) vector[i]);		\
+
+template<>
+void SGVector<complex128_t>::save(CFile* saver)
+{
+	SG_SERROR("SGVector::save():: Not supported for complex128_t\n");
 }
 
-MATHOP(abs)
-MATHOP(acos)
-MATHOP(asin)
-MATHOP(atan)
-MATHOP(cos)
-MATHOP(cosh)
-MATHOP(exp)
-MATHOP(log)
-MATHOP(log10)
-MATHOP(sin)
-MATHOP(sinh)
-MATHOP(sqrt)
-MATHOP(tan)
-MATHOP(tanh)
-#undef MATHOP
-		
-template <class T> void SGVector<T>::atan2(T x)
+template <class T> SGVector<float64_t> SGVector<T>::get_real()
 {
+	SGVector<float64_t> real(vlen);
 	for (int32_t i=0; i<vlen; i++)
-		vector[i]=CMath::atan2(vector[i], x);
+		real[i]=CMath::real(vector[i]);
+	return real;
 }
-		
-template <class T> void SGVector<T>::pow(T q)
+
+template <class T> SGVector<float64_t> SGVector<T>::get_imag()
 {
+	SGVector<float64_t> imag(vlen);
 	for (int32_t i=0; i<vlen; i++)
-		vector[i]=(T) CMath::pow((double) vector[i], (double) q);
+		imag[i]=CMath::imag(vector[i]);
+	return imag;
 }
 
+template <class T>
+SGMatrix<T> SGVector<T>::convert_to_matrix(SGVector<T> vector,
+	index_t nrows, index_t ncols, bool fortran_order)
+{
+	if (nrows*ncols>vector.size())
+		SG_SERROR("SGVector::convert_to_matrix():: Dimensions mismatch\n");
+
+	T* data=NULL;
+	SGVector<T>::convert_to_matrix(data, nrows, ncols, vector.vector, vector.vlen, fortran_order);
+
+	SGMatrix<T> matrix=SGMatrix<T>(data, nrows, ncols);
+	return matrix;
+}
+
+template <class T>
+void SGVector<T>::convert_to_matrix(T*& matrix, index_t nrows, index_t ncols, const T* vector, int32_t vlen, bool fortran_order)
+{
+	if (nrows*ncols>vlen)
+		SG_SERROR("SGVector::convert_to_matrix():: Dimensions mismatch\n");
+
+	if (matrix!=NULL)
+		SG_FREE(matrix);
+	matrix=SG_MALLOC(T, nrows*ncols);
+
+	if (fortran_order)
+	{
+		for (index_t i=0; i<ncols*nrows; i++)
+			matrix[i]=vector[i];
+	}
+	else
+	{
+		for (index_t i=0; i<nrows; i++)
+		{
+			for (index_t j=0; j<ncols; j++)
+				matrix[i+j*nrows]=vector[j+i*ncols];
+		}
+	}
+}
+
+#define UNDEFINED(function, type)	\
+template <>	\
+SGVector<float64_t> SGVector<type>::function()	\
+{	\
+	SG_SERROR("SGVector::%s():: Not supported for %s\n",	\
+		#function, #type);	\
+	SGVector<float64_t> ret(vlen);	\
+	return ret;	\
+}
+
+UNDEFINED(get_real, bool)
+UNDEFINED(get_real, char)
+UNDEFINED(get_real, int8_t)
+UNDEFINED(get_real, uint8_t)
+UNDEFINED(get_real, int16_t)
+UNDEFINED(get_real, uint16_t)
+UNDEFINED(get_real, int32_t)
+UNDEFINED(get_real, uint32_t)
+UNDEFINED(get_real, int64_t)
+UNDEFINED(get_real, uint64_t)
+UNDEFINED(get_real, float32_t)
+UNDEFINED(get_real, float64_t)
+UNDEFINED(get_real, floatmax_t)
+UNDEFINED(get_imag, bool)
+UNDEFINED(get_imag, char)
+UNDEFINED(get_imag, int8_t)
+UNDEFINED(get_imag, uint8_t)
+UNDEFINED(get_imag, int16_t)
+UNDEFINED(get_imag, uint16_t)
+UNDEFINED(get_imag, int32_t)
+UNDEFINED(get_imag, uint32_t)
+UNDEFINED(get_imag, int64_t)
+UNDEFINED(get_imag, uint64_t)
+UNDEFINED(get_imag, float32_t)
+UNDEFINED(get_imag, float64_t)
+UNDEFINED(get_imag, floatmax_t)
+#undef UNDEFINED
+
 template class SGVector<bool>;
 template class SGVector<char>;
 template class SGVector<int8_t>;
@@ -1042,4 +987,10 @@ template class SGVector<uint64_t>;
 template class SGVector<float32_t>;
 template class SGVector<float64_t>;
 template class SGVector<floatmax_t>;
+template class SGVector<complex128_t>;
 }
+
+#undef COMPLEX128_ERROR_NOARG
+#undef COMPLEX128_ERROR_ONEARG
+#undef COMPLEX128_ERROR_TWOARGS
+#undef COMPLEX128_ERROR_THREEARGS
