@@ -5,26 +5,28 @@
  * (at your option) any later version.
  *
  * Written (W) 2012 Sergey Lisitsyn
- * Copyright (C) 2010-2012 Jun Liu, Jieping Ye 
+ * Copyright (C) 2010-2012 Jun Liu, Jieping Ye
  */
 
 #ifndef  SLEP_MT_LOGISTIC_H_
 #define  SLEP_MT_LOGISTIC_H_
 
+#include <shogun/lib/config.h>
+
 #include <shogun/lib/slep/slep_options.h>
 #include <shogun/features/DotFeatures.h>
 
-namespace shogun 
+namespace shogun
 {
 
-/** Learning optimization task solver ported from the 
+/** Learning optimization task solver ported from the
  * SLEP (Sparse LEarning Package) library.
- * 
+ *
  * Based on accelerated projected gradient method.
  *
  * Supports two types of losses: logistic and least squares.
  *
- * Supports multitask problems (task group [MULTITASK_GROUP] 
+ * Supports multitask problems (task group [MULTITASK_GROUP]
  * and task tree [MULTITASK_TREE] relations),
  * problems with feature relations (feature group [FEATURE_GROUP]
  * and feature tree [FEATURE_TREE]),
