@@ -12,9 +12,13 @@
 #include <shogun/machine/KernelMachine.h>
 #include <shogun/lib/Signal.h>
 #include <shogun/labels/RegressionLabels.h>
-#include <shogun/base/Parameter.h>
+#include <shogun/io/SGIO.h>
 #include <shogun/base/ParameterMap.h>
 
+#include <shogun/kernel/Kernel.h>
+#include <shogun/kernel/CustomKernel.h>
+#include <shogun/labels/Labels.h>
+
 using namespace shogun;
 
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
@@ -621,6 +625,8 @@ void CKernelMachine::data_lock(CLabels* labs, CFeatures* features)
 {
 	if ( !kernel )
 		SG_ERROR("The kernel is not initialized\n")
+	if (kernel->has_property(KP_KERNCOMBINATION))
+		SG_ERROR("Locking is not supported (yet) with combined kernel. Please disable it in cross validation")
 
 	/* init kernel with data */
 	kernel->init(features, features);
@@ -704,11 +710,6 @@ void CKernelMachine::init()
 	m_parameter_map->finalize_map();
 }
 
-CMachine* CKernelMachine::clone()
-{
-	return new CKernelMachine(this);
-}
-
 bool CKernelMachine::supports_locking() const
 {
 	return true;
