@@ -9,6 +9,7 @@
  */
 
 #include <shogun/machine/KernelStructuredOutputMachine.h>
+#include <shogun/kernel/Kernel.h>
 
 using namespace shogun;
 
@@ -20,10 +21,9 @@ CKernelStructuredOutputMachine::CKernelStructuredOutputMachine()
 
 CKernelStructuredOutputMachine::CKernelStructuredOutputMachine(
 		CStructuredModel*  model,
-		CLossFunction*     loss,
 		CStructuredLabels* labs,
 		CKernel*           kernel)
-: CStructuredOutputMachine(model, loss, labs), m_kernel(NULL)
+: CStructuredOutputMachine(model, labs), m_kernel(NULL)
 {
 	set_kernel(kernel);
 	register_parameters();
