@@ -4,37 +4,39 @@
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
- * Written (W) 2011-2012 Heiko Strathmann
+ * Written (W) 2014 Wu Lin
+ * Written (W) 2011-2013 Heiko Strathmann
  * Copyright (C) 2011 Berlin Institute of Technology and Max-Planck-Society
  *
  * ALGLIB Copyright 1984, 1987, 1995, 2000 by Stephen L. Moshier under GPL2+
  * http://www.alglib.net/
  * See header file for which functions are taken from ALGLIB (with adjustments
  * for shogun)
+ *
+ * lnormal_cdf and normal_cdf are adapted from
+ * Gaussian Process Machine Learning Toolbox file logphi.m
+ * http://www.gaussianprocess.org/gpml/code/matlab/doc/
+ * and PraatLib 0.3 (GPL v2) file gsl_specfunc__erfc.c
  */
 
 #include <shogun/mathematics/Statistics.h>
 #include <shogun/mathematics/Math.h>
 #include <shogun/lib/SGMatrix.h>
 #include <shogun/lib/SGVector.h>
+#include <shogun/lib/SGSparseMatrix.h>
+#include <shogun/lib/SGSparseVector.h>
 
 #ifdef HAVE_LAPACK
 #include <shogun/mathematics/lapack.h>
 #endif //HAVE_LAPACK
+
+#ifdef HAVE_EIGEN3
+#include <shogun/mathematics/eigen3.h>
+using namespace Eigen;
+#endif //HAVE_EIGEN3
+
 using namespace shogun;
 
-float64_t CStatistics::mean(SGVector<float64_t> values)
-{
-	ASSERT(values.vlen>0)
-	ASSERT(values.vector)
-
-	float64_t sum=0;
-	for (index_t i=0; i<values.vlen; ++i)
-		sum+=values.vector[i];
-
-	return sum/values.vlen;
-}
-
 float64_t CStatistics::median(SGVector<float64_t> values, bool modify,
 			bool in_place)
 {
@@ -1678,9 +1680,132 @@ float64_t CStatistics::inverse_incomplete_gamma_completed(float64_t a,
 
 float64_t CStatistics::normal_cdf(float64_t x, float64_t std_dev)
 {
-	return 0.5*(error_function(x/std_dev/1.41421356237309504880)+1);
+	return 0.5*(error_function_complement(-x/std_dev/1.41421356237309514547));
 }
 
+
+const float64_t CStatistics::ERFC_CASE1=0.0492;
+
+const float64_t CStatistics::ERFC_CASE2=-11.3137;
+
+float64_t CStatistics::lnormal_cdf(float64_t x)
+{
+	const float64_t sqrt_of_2=1.41421356237309514547;
+	const float64_t log_of_2=0.69314718055994528623;
+	const float64_t sqrt_of_pi=1.77245385090551588192;
+
+	const index_t c_len=14;
+	static float64_t c_array[c_len]=
+	{
+		0.00048204,
+		-0.00142906,
+		0.0013200243174,
+		0.0009461589032,
+		-0.0045563339802,
+		0.00556964649138,
+		0.00125993961762116,
+		-0.01621575378835404,
+		0.02629651521057465,
+		-0.001829764677455021,
+		2.0*(1.0-CMath::PI/3.0),
+		(4.0-CMath::PI)/3.0,
+		1.0,
+		1.0
+	};
+
+	if (x*x<ERFC_CASE1)
+	{
+		//id1 = z.*z<0.0492;                               
+		//lp0 = -z(id1)/sqrt(2*pi);
+		//c = [ 0.00048204; -0.00142906; 0.0013200243174; 0.0009461589032;
+		//-0.0045563339802; 0.00556964649138; 0.00125993961762116;
+		//-0.01621575378835404; 0.02629651521057465; -0.001829764677455021;
+		//2*(1-pi/3); (4-pi)/3; 1; 1];
+		//f = 0; for i=1:14, f = lp0.*(c(i)+f); end, lp(id1) = -2*f-log(2);
+		float64_t f = 0.0;
+		float64_t lp0 = -x/(sqrt_of_2*sqrt_of_pi);
+		for (index_t i=0; i<c_len; i++)
+			f=lp0*(c_array[i]+f);
+		return -2.0*f-log_of_2;
+	}
+	else if (x<ERFC_CASE2)
+	{
+		//id2 = z<-11.3137;                           
+		//r = [ 1.2753666447299659525; 5.019049726784267463450;
+		//6.1602098531096305441; 7.409740605964741794425;
+		//2.9788656263939928886 ];
+		//q = [ 2.260528520767326969592;  9.3960340162350541504;
+		//12.048951927855129036034; 17.081440747466004316; 
+		//9.608965327192787870698;  3.3690752069827527677 ];
+		//num = 0.5641895835477550741; for i=1:5, num = -z(id2).*num/sqrt(2) + r(i); end
+		//den = 1.0;                   for i=1:6, den = -z(id2).*den/sqrt(2) + q(i); end
+		//e = num./den; lp(id2) = log(e/2) - z(id2).^2/2;
+
+		return CMath::log(erfc8_weighted_sum(x))-log_of_2-x*x*0.5;
+	}
+
+	//id3 = ~id2 & ~id1; lp(id3) = log(erfc(-z(id3)/sqrt(2))/2);
+	return CMath::log(normal_cdf(x));
+}
+
+float64_t CStatistics::chi2_cdf(float64_t x, float64_t k)
+{
+	/* F(x,k) = incomplete_gamma(k/2,x/2) divided by true gamma(k/2) */
+	return incomplete_gamma(k/2.0,x/2.0);
+}
+
+float64_t CStatistics::fdistribution_cdf(float64_t x, float64_t d1, float64_t d2)
+{
+	/* F(x;d1,d2) = incomplete_beta(d1/2, d2/2, d1*x/(d1*x+d2)) divided by beta(d1/2,d2/2)*/
+	return incomplete_beta(d1/2.0,d2/2.0,d1*x/(d1*x+d2));
+}
+
+float64_t CStatistics::erfc8_weighted_sum(float64_t x)
+{
+	/* This is based on index 5725 in Hart et al */
+
+	const float64_t sqrt_of_2=1.41421356237309514547;
+
+	static float64_t P[]=
+	{
+		0.5641895835477550741253201704,
+		1.275366644729965952479585264,
+		5.019049726784267463450058,
+		6.1602098531096305440906,
+		7.409740605964741794425,
+		2.97886562639399288862
+	};
+
+	static float64_t Q[]=
+	{
+		1.0,
+		2.260528520767326969591866945,
+		9.396034016235054150430579648,
+		12.0489519278551290360340491,
+		17.08144074746600431571095,
+		9.608965327192787870698,
+		3.3690752069827527677
+	};
+
+	float64_t num=0.0, den=0.0;
+
+	num = P[0];
+	for (index_t i=1; i<6; i++)
+	{
+		num=-x*num/sqrt_of_2+P[i];
+	}
+
+	den = Q[0];
+	for (index_t i=1; i<7; i++)
+	{
+		den=-x*den/sqrt_of_2+Q[i];
+	}
+
+	return num/den;
+}
+
+
+
 float64_t CStatistics::error_function(float64_t x)
 {
 	float64_t xsq;
@@ -1793,7 +1918,7 @@ float64_t CStatistics::fishers_exact_test_for_2x3_table(
 	m[4]=table.matrix[4]+table.matrix[5];
 
 	float64_t n=SGVector<float64_t>::sum(m, m_len)/2.0;
-	int32_t x_len=2*3*CMath::sq(SGVector<float64_t>::max(m, m_len));
+	int32_t x_len=2*3*CMath::sq(CMath::max(m, m_len));
 	float64_t* x=SG_MALLOC(float64_t, x_len);
 	SGVector<float64_t>::fill_vector(x, x_len, 0.0);
 
@@ -1939,17 +2064,437 @@ SGVector<int32_t> CStatistics::sample_indices(int32_t sample_size, int32_t N)
 	SG_FREE(idxs);
 
 	SGVector<int32_t> result=SGVector<int32_t>(permuted_idxs, sample_size);
-	result.qsort();
+	CMath::qsort(result);
 	return result;
 }
 
 float64_t CStatistics::dlgamma(float64_t x)
 {
-	x = x+6.0;
-        float64_t df = 1./(x*x);
-        df = (((df/240-0.003968253986254)*df+1/120.0)*df-1/120.0)*df;
-        df = df+log(x)-0.5/x-1.0/(x-1.0)-1.0/(x-2.0)-1.0/
-                          (x-3.0)-1.0/(x-4.0)-1.0/(x-5.0)-1.0/(x-6.0);
-        return df;
+	float64_t result=0.0;
+
+	if (x<0.0)
+	{
+		// use reflection formula
+		x=1.0-x;
+		result=CMath::PI/CMath::tan(CMath::PI*x);
+	}
+
+	// make x>7 for approximation
+	// (use reccurent formula: psi(x+1) = psi(x) + 1/x)
+	while (x<=7.0)
+	{
+		result-=1.0/x;
+		x++;
+	}
+
+	// perform approximation
+	x-=0.5;
+	result+=log(x);
+
+	float64_t coeff[10]={
+		0.04166666666666666667,
+		-0.00729166666666666667,
+		0.00384424603174603175,
+		-0.00413411458333333333,
+		0.00756096117424242424,
+		-0.02108249687595390720,
+		0.08332316080729166666,
+		-0.44324627670587277880,
+		3.05393103044765369366,
+		-26.45616165999210241989};
+
+	float64_t power=1.0;
+	float64_t ix2=1.0/CMath::sq(x);
+
+	// perform approximation
+	for (index_t i=0; i<10; i++)
+	{
+		power*=ix2;
+		result+=coeff[i]*power;
+	}
+
+	return result;
 }
 
+#ifdef HAVE_EIGEN3
+float64_t CStatistics::log_det_general(const SGMatrix<float64_t> A)
+{
+	Map<MatrixXd> eigen_A(A.matrix, A.num_rows, A.num_cols);
+	REQUIRE(eigen_A.rows()==eigen_A.cols(),
+		"Input matrix should be a sqaure matrix row(%d) col(%d)\n",
+		eigen_A.rows(), eigen_A.cols());
+
+	PartialPivLU<MatrixXd> lu(eigen_A);
+	VectorXd tmp(eigen_A.rows());
+
+	for (index_t idx=0; idx<tmp.rows(); idx++)
+		tmp[idx]=idx+1;
+
+	VectorXd p=lu.permutationP()*tmp;
+	int detP=1;
+
+	for (index_t idx=0; idx<p.rows(); idx++)
+	{
+		if (p[idx]!=idx+1)
+		{
+			detP*=-1;
+			index_t j=idx+1;
+			while(j<p.rows())
+			{
+				if (p[j]==idx+1)
+					break;
+				j++;
+			}
+			p[j]=p[idx];
+		}
+	}
+
+	VectorXd u=lu.matrixLU().diagonal();
+	int check_u=1;
+
+	for (int idx=0; idx<u.rows(); idx++)
+	{
+		if (u[idx]<0)
+			check_u*=-1;
+		else if (u[idx]==0)
+		{
+			check_u=0;
+			break;
+		}
+	}
+
+	float64_t result=CMath::INFTY;
+
+	if (check_u==detP)
+		result=u.array().abs().log().sum();
+
+	return result;
+}
+
+float64_t CStatistics::log_det(SGMatrix<float64_t> m)
+{
+	/* map the matrix to eigen3 to perform cholesky */
+	Map<MatrixXd> M(m.matrix, m.num_rows, m.num_cols);
+
+	/* computing the cholesky decomposition */
+	LLT<MatrixXd> llt;
+	llt.compute(M);
+
+	/* the lower triangular matrix */
+	MatrixXd l = llt.matrixL();
+
+	/* calculate the log-determinant */
+	VectorXd diag = l.diagonal();
+	float64_t retval = 0.0;
+	for( int32_t i = 0; i < diag.rows(); ++i ) {
+		retval += log(diag(i));
+	}
+	retval *= 2;
+
+	return retval;
+}
+
+float64_t CStatistics::log_det(const SGSparseMatrix<float64_t> m)
+{
+	typedef SparseMatrix<float64_t> MatrixType;
+	const MatrixType &M=EigenSparseUtil<float64_t>::toEigenSparse(m);
+
+	SimplicialLLT<MatrixType> llt;
+
+	// factorize using cholesky with amd permutation
+	llt.compute(M);
+	MatrixType L=llt.matrixL();
+
+	// calculate the log-determinant
+	float64_t retval=0.0;
+	for( index_t i=0; i<M.rows(); ++i )
+		retval+=log(L.coeff(i,i));
+	retval*=2;
+
+	return retval;
+}
+
+SGMatrix<float64_t> CStatistics::sample_from_gaussian(SGVector<float64_t> mean,
+	SGMatrix<float64_t> cov, int32_t N, bool precision_matrix)
+{
+	REQUIRE(cov.num_rows>0, "Number of covariance rows must be positive!\n");
+	REQUIRE(cov.num_cols>0,"Number of covariance cols must be positive!\n");
+	REQUIRE(cov.matrix, "Covariance is not initialized!\n");
+	REQUIRE(cov.num_rows==cov.num_cols, "Covariance should be square matrix!\n");
+	REQUIRE(mean.vlen==cov.num_rows, "Mean and covariance dimension mismatch!\n");
+
+	int32_t dim=mean.vlen;
+	Map<VectorXd> mu(mean.vector, mean.vlen);
+	Map<MatrixXd> c(cov.matrix, cov.num_rows, cov.num_cols);
+
+	// generate samples, z,  from N(0, I), DxN
+	SGMatrix<float64_t> S(dim, N);
+	for( int32_t j=0; j<N; ++j )
+		for( int32_t i=0; i<dim; ++i )
+			S(i,j)=CMath::randn_double();
+
+	// the cholesky factorization c=L*U
+	MatrixXd U=c.llt().matrixU();
+
+	// generate samples, x, from N(mean, cov) or N(mean, cov^-1)
+	// return samples of dimension NxD
+	if( precision_matrix )
+	{
+		// here we have U*x=z, to solve this, we use cholesky again
+		Map<MatrixXd> s(S.matrix, S.num_rows, S.num_cols);
+		LDLT<MatrixXd> ldlt;
+		ldlt.compute(U);
+		s=ldlt.solve(s);
+	}
+
+	SGMatrix<float64_t>::transpose_matrix(S.matrix, S.num_rows, S.num_cols);
+
+	if( !precision_matrix )
+	{
+		// here we need to find x=L*z, so, x'=z'*L' i.e. x'=z'*U
+		Map<MatrixXd> s(S.matrix, S.num_rows, S.num_cols);
+		s=s*U;
+	}
+
+	// add the mean
+	Map<MatrixXd> x(S.matrix, S.num_rows, S.num_cols);
+	for( int32_t i=0; i<N; ++i )
+		x.row(i)+=mu;
+
+	return S;
+}
+
+SGMatrix<float64_t> CStatistics::sample_from_gaussian(SGVector<float64_t> mean,
+ SGSparseMatrix<float64_t> cov, int32_t N, bool precision_matrix)
+{
+	REQUIRE(cov.num_vectors>0,
+		"CStatistics::sample_from_gaussian(): \
+		Number of covariance rows must be positive!\n");
+	REQUIRE(cov.num_features>0,
+		"CStatistics::sample_from_gaussian(): \
+		Number of covariance cols must be positive!\n");
+	REQUIRE(cov.sparse_matrix,
+		"CStatistics::sample_from_gaussian(): \
+		Covariance is not initialized!\n");
+	REQUIRE(cov.num_vectors==cov.num_features,
+		"CStatistics::sample_from_gaussian(): \
+		Covariance should be square matrix!\n");
+	REQUIRE(mean.vlen==cov.num_vectors,
+		"CStatistics::sample_from_gaussian(): \
+		Mean and covariance dimension mismatch!\n");
+
+	int32_t dim=mean.vlen;
+	Map<VectorXd> mu(mean.vector, mean.vlen);
+
+	typedef SparseMatrix<float64_t> MatrixType;
+	const MatrixType &c=EigenSparseUtil<float64_t>::toEigenSparse(cov);
+
+	SimplicialLLT<MatrixType> llt;
+
+	// generate samples, z,  from N(0, I), DxN
+	SGMatrix<float64_t> S(dim, N);
+	for( int32_t j=0; j<N; ++j )
+		for( int32_t i=0; i<dim; ++i )
+			S(i,j)=CMath::randn_double();
+
+	Map<MatrixXd> s(S.matrix, S.num_rows, S.num_cols);
+
+	// the cholesky factorization P*c*P^-1 = LP*UP, with LP=P*L, UP=U*P^-1
+	llt.compute(c);
+	MatrixType LP=llt.matrixL();
+	MatrixType UP=llt.matrixU();
+
+	// generate samples, x, from N(mean, cov) or N(mean, cov^-1)
+	// return samples of dimension NxD
+	if( precision_matrix )
+	{
+		// here we have UP*xP=z, to solve this, we use cholesky again
+		SimplicialLLT<MatrixType> lltUP;
+		lltUP.compute(UP);
+		s=lltUP.solve(s);
+	}
+	else
+	{
+		// here we need to find xP=LP*z
+		s=LP*s;
+	}
+
+	// permute the samples back with x=P^-1*xP
+	s=llt.permutationPinv()*s;
+
+	SGMatrix<float64_t>::transpose_matrix(S.matrix, S.num_rows, S.num_cols);
+	// add the mean
+	Map<MatrixXd> x(S.matrix, S.num_rows, S.num_cols);
+	for( int32_t i=0; i<N; ++i )
+		x.row(i)+=mu;
+
+	return S;
+}
+
+#endif //HAVE_EIGEN3
+
+CStatistics::SigmoidParamters CStatistics::fit_sigmoid(SGVector<float64_t> scores)
+{
+	SG_SDEBUG("entering CStatistics::fit_sigmoid()\n")
+
+	REQUIRE(scores.vector, "CStatistics::fit_sigmoid() requires "
+			"scores vector!\n");
+
+	/* count prior0 and prior1 if needed */
+	int32_t prior0=0;
+	int32_t prior1=0;
+	SG_SDEBUG("counting number of positive and negative labels\n")
+	{
+		for (index_t i=0; i<scores.vlen; ++i)
+		{
+			if (scores[i]>0)
+				prior1++;
+			else
+				prior0++;
+		}
+	}
+	SG_SDEBUG("%d pos; %d neg\n", prior1, prior0)
+
+	/* parameter setting */
+	/* maximum number of iterations */
+	index_t maxiter=100;
+
+	/* minimum step taken in line search */
+	float64_t minstep=1E-10;
+
+	/* for numerically strict pd of hessian */
+	float64_t sigma=1E-12;
+	float64_t eps=1E-5;
+
+	/* construct target support */
+	float64_t hiTarget=(prior1+1.0)/(prior1+2.0);
+	float64_t loTarget=1/(prior0+2.0);
+	index_t length=prior1+prior0;
+
+	SGVector<float64_t> t(length);
+	for (index_t i=0; i<length; ++i)
+	{
+		if (scores[i]>0)
+			t[i]=hiTarget;
+		else
+			t[i]=loTarget;
+	}
+
+	/* initial Point and Initial Fun Value */
+	/* result parameters of sigmoid */
+	float64_t a=0;
+	float64_t b=CMath::log((prior0+1.0)/(prior1+1.0));
+	float64_t fval=0.0;
+
+	for (index_t i=0; i<length; ++i)
+	{
+		float64_t fApB=scores[i]*a+b;
+		if (fApB>=0)
+			fval+=t[i]*fApB+CMath::log(1+CMath::exp(-fApB));
+		else
+			fval+=(t[i]-1)*fApB+CMath::log(1+CMath::exp(fApB));
+	}
+
+	index_t it;
+	float64_t g1;
+	float64_t g2;
+	for (it=0; it<maxiter; ++it)
+	{
+		SG_SDEBUG("Iteration %d, a=%f, b=%f, fval=%f\n", it, a, b, fval)
+
+		/* Update Gradient and Hessian (use H' = H + sigma I) */
+		float64_t h11=sigma; //Numerically ensures strict PD
+		float64_t h22=h11;
+		float64_t h21=0;
+		g1=0;
+		g2=0;
+
+		for (index_t i=0; i<length; ++i)
+		{
+			float64_t fApB=scores[i]*a+b;
+			float64_t p;
+			float64_t q;
+			if (fApB>=0)
+			{
+				p=CMath::exp(-fApB)/(1.0+CMath::exp(-fApB));
+				q=1.0/(1.0+CMath::exp(-fApB));
+			}
+			else
+			{
+				p=1.0/(1.0+CMath::exp(fApB));
+				q=CMath::exp(fApB)/(1.0+CMath::exp(fApB));
+			}
+
+			float64_t d2=p*q;
+			h11+=scores[i]*scores[i]*d2;
+			h22+=d2;
+			h21+=scores[i]*d2;
+			float64_t d1=t[i]-p;
+			g1+=scores[i]*d1;
+			g2+=d1;
+		}
+
+		/* Stopping Criteria */
+		if (CMath::abs(g1)<eps && CMath::abs(g2)<eps)
+			break;
+
+		/* Finding Newton direction: -inv(H') * g */
+		float64_t det=h11*h22-h21*h21;
+		float64_t dA=-(h22*g1-h21*g2)/det;
+		float64_t dB=-(-h21*g1+h11*g2)/det;
+		float64_t gd=g1*dA+g2*dB;
+
+		/* Line Search */
+		float64_t stepsize=1;
+
+		while (stepsize>=minstep)
+		{
+			float64_t newA=a+stepsize*dA;
+			float64_t newB=b+stepsize*dB;
+
+			/* New function value */
+			float64_t newf=0.0;
+			for (index_t i=0; i<length; ++i)
+			{
+				float64_t fApB=scores[i]*newA+newB;
+				if (fApB>=0)
+					newf+=t[i]*fApB+CMath::log(1+CMath::exp(-fApB));
+				else
+					newf+=(t[i]-1)*fApB+CMath::log(1+CMath::exp(fApB));
+			}
+
+			/* Check sufficient decrease */
+			if (newf<fval+0.0001*stepsize*gd)
+			{
+				a=newA;
+				b=newB;
+				fval=newf;
+				break;
+			}
+			else
+				stepsize=stepsize/2.0;
+		}
+
+		if (stepsize<minstep)
+		{
+			SG_SWARNING("CStatistics::fit_sigmoid(): line search fails, A=%f, "
+					"B=%f, g1=%f, g2=%f, dA=%f, dB=%f, gd=%f\n",
+					a, b, g1, g2, dA, dB, gd);
+		}
+	}
+
+	if (it>=maxiter-1)
+	{
+		SG_SWARNING("CStatistics::fit_sigmoid(): reaching maximal iterations,"
+				" g1=%f, g2=%f\n", g1, g2);
+	}
+
+	SG_SDEBUG("fitted sigmoid: a=%f, b=%f\n", a, b)
+
+	CStatistics::SigmoidParamters result;
+	result.a=a;
+	result.b=b;
+
+	SG_SDEBUG("leaving CStatistics::fit_sigmoid()\n")
+	return result;
+}
