@@ -16,13 +16,15 @@
 #ifndef __STATISTICS_H_
 #define __STATISTICS_H_
 
-#include <math.h>
 #include <shogun/lib/config.h>
 #include <shogun/base/SGObject.h>
+#include <shogun/lib/SGVector.h>
+#include <shogun/io/SGIO.h>
 
 namespace shogun
 {
 template<class T> class SGMatrix;
+template<class T> class SGSparseMatrix;
 
 /** @brief Class that contains certain functions related to statistics, such as
  * probability/cumulative distribution functions, different statistics, etc.
@@ -35,10 +37,19 @@ public:
 	/** Calculates mean of given values. Given \f$\{x_1, ..., x_m\}\f$, this
 	 * is \f$\frac{1}{m}\sum_{i=1}^m x_i\f$
 	 *
-	 * @param values vector of values
+	 * @param vec vector of values
 	 * @return mean of given values
 	 */
-	static float64_t mean(SGVector<float64_t> values);
+	template<class T>
+		static floatmax_t mean(SGVector<T> vec)
+		{
+			floatmax_t sum = 0;
+
+			for ( index_t i = 0 ; i < vec.vlen ; ++i )
+				sum += vec[i];
+
+			return sum/vec.vlen;
+		}
 
 	/** Calculates median of given values. The median is the value that one
 	 * gets when the input vector is sorted and then selects the middle value.
@@ -200,7 +211,7 @@ public:
 	 */
 	static float64_t inverse_student_t(int32_t k, float64_t p);
 
-	/** Inverse of imcomplete beta integral
+	/** Inverse of incomplete beta integral
 	 *
 	 * Given \f$y\f$, the function finds \f$x\f$ such that
 	 *
@@ -340,7 +351,7 @@ public:
 	 */
 	static float64_t inverse_gamma_cdf(float64_t p, float64_t a, float64_t b);
 
-	/** Inverse of complemented imcomplete gamma integral
+	/** Inverse of complemented incomplete gamma integral
 	 *
 	 * Given \f$p\f$, the function finds \f$x\f$ such that
 	 *
@@ -376,6 +387,49 @@ public:
 	 */
 	static float64_t normal_cdf(float64_t x, float64_t std_dev=1);
 
+	/** returns logarithm of the cumulative distribution function
+	 * (CDF) of Gaussian distribution \f$N(0, 1)\f$:
+	 *
+	 * \f[
+	 * \text{lnormal\_cdf}(x)=log\left(\frac{1}{2}+
+	 * \frac{1}{2}\text{error\_function}(\frac{x}{\sqrt{2}})\right)
+	 * \f]
+	 *
+	 * This method uses asymptotic expansion for \f$x<-10.0\f$,
+	 * otherwise it returns \f$log(\text{normal\_cdf}(x))\f$.
+	 *
+	 * @param x real value
+	 *
+	 * @return \f$log(\text{normal\_cdf}(x))\f$
+	 */
+	static float64_t lnormal_cdf(float64_t x);
+
+	/** Evaluates the CDF of the chi square distribution with 
+	 * parameter k at \f$x\f$. Based on Wikipedia definition.
+	 *
+	 * @param x position to evaluate
+	 * @param k parameter
+	 * @return chi square CDF at \f$x\f$
+	 */
+	static float64_t chi2_cdf(float64_t x, float64_t k);
+
+	/** Evaluates the CDF of the F-distribution with parameters
+	 * \f$d1,d2\f$ at \f$x\f$. Based on Wikipedia definition.
+	 *
+	 * @param x position to evaluate
+	 * @param d1 parameter 1
+	 * @param d2 parameter 2
+	 * @return F-distribution CDF at \f$x\f$
+	 */
+	static float64_t fdistribution_cdf(float64_t x, float64_t d1, float64_t d2);
+
+	/** Use to estimates erfc(x) valid for -100 < x < -8
+	 *
+	 * @param x real value
+	 * @return weighted sum
+	 */
+	static float64_t erfc8_weighted_sum(float64_t x);
+
 	/** Error function
 	 *
 	 * The integral is
@@ -444,15 +498,129 @@ public:
 	}
 
 	 /** Derivative of the log gamma function.
-         *
-         * Taken from likT.m from the GPML
-         * toolbox.
-         *
-         * @param x input
-         * @return derivative of the log gamma input
-         */
-        static float64_t dlgamma(float64_t x);
+	 *
+	 * @param x input
+	 * @return derivative of the log gamma input
+	 */
+	static float64_t dlgamma(float64_t x);
 
+	/** Representation of a Sigmoid function for the fit_sigmoid function */
+	struct SigmoidParamters
+	{
+		/** parameter a */
+		float64_t a;
+
+		/** parameter b */
+		float64_t b;
+	};
+
+	/** Converts a given vector of scores to calibrated probabilities by fitting a
+	 * sigmoid function using the method described in
+	 * Lin, H., Lin, C., and Weng, R. (2007).
+	 * A note on Platt's probabilistic outputs for support vector machines.
+	 *
+	 * This can be used to transform scores to probabilities as setting
+	 * \f$pf=x*a+b\f$ for a given score \f$x\f$ and computing
+	 * \f$\frac{\exp(-f)}{1+}exp(-f)}\f$ if \f$f\geq 0\f$ and
+	 * \f$\frac{1}{(1+\exp(f)}\f$ otherwise
+	 *
+	 * @param scores scores to fit the sigmoid to
+	 * @return struct containing the sigmoid's shape parameters a and b
+	 */
+	static SigmoidParamters fit_sigmoid(SGVector<float64_t> scores);
+
+#ifdef HAVE_EIGEN3
+	/** The log determinant of a dense matrix
+	 *
+	 * If determinant of the input matrix is positive, it returns the logarithm of the value.
+	 * If not, it returns CMath::INFTY 
+	 * Note that the input matrix is not required to be symmetric positive definite.
+	 * This method is slower than log_det() if input matrix is known to be symmetric positive definite
+	 *
+	 * It is adapted from Gaussian Process Machine Learning Toolbox
+	 * http://www.gaussianprocess.org/gpml/code/matlab/doc/
+	 *
+	 * @param A input matrix
+	 * @return the log determinant value
+	 */
+
+	static float64_t log_det_general(const SGMatrix<float64_t> A);
+
+	/** The log determinant of a dense matrix
+	 *
+	 * The log determinant of a positive definite symmetric real valued
+	 * matrix is calculated as
+	 * \f[
+	 * \text{log\_determinant}(M)
+	 * = \text{log}(\text{determinant}(L)\times\text{determinant}(L'))
+	 * = 2\times \sum_{i}\text{log}(L_{i,i})
+	 * \f]
+	 * Where, \f$M = L\times L'\f$ as per Cholesky decomposition.
+	 *
+	 * @param m input matrix
+	 * @return the log determinant value
+	 */
+	static float64_t log_det(SGMatrix<float64_t> m);
+
+	/** The log determinant of a sparse matrix
+	 *
+	 * The log determinant of symmetric positive definite sparse matrix
+	 * is calculated in a similar way as the dense case. But using
+	 * cholesky decomposition on sparse matrices may suffer from fill-in
+	 * phenomenon, i.e. the factors may not be as sparse. The
+	 * SimplicialCholesky module for sparse matrix in eigen3 library
+	 * uses an approach called approximate minimum degree reordering,
+	 * or amd, which permutes the matrix beforehand and results in much
+	 * sparser factors. If \f$P\f$ is the permutation matrix, it computes
+	 * \f$\text{LLT}(P\times M\times P^{-1}) = L\times L'\f$.
+	 *
+	 * @param m input sparse matrix
+	 * @return the log determinant value
+	 */
+	static float64_t log_det(const SGSparseMatrix<float64_t> m);
+
+	/** Sampling from a multivariate Gaussian distribution with
+	 * dense covariance matrix
+	 *
+	 * Sampling is performed by taking samples from \f$N(0, I)\f$, then
+	 * using cholesky factor of the covariance matrix, \f$\Sigma\f$ and
+	 * performing
+	 * \f[S_{N(\mu,\Sigma)}=S_{N(0,I)}*L^{T}+\mu\f]
+	 * where \f$\Sigma=L*L^{T}\f$ and \f$\mu\f$ is the mean vector.
+	 *
+	 * @param mean the mean vector
+	 * @param cov the covariance matrix
+	 * @param N number of samples
+	 * @param precision_matrix if true, sample from N(mu,C^-1)
+	 * @return the sample matrix of size \f$N\times dim\f$
+	 */
+	static SGMatrix<float64_t> sample_from_gaussian(SGVector<float64_t> mean,
+	SGMatrix<float64_t> cov, int32_t N=1, bool precision_matrix=false);
+
+	/** Sampling from a multivariate Gaussian distribution with
+	 * sparse covariance matrix
+	 *
+	 * Sampling is performed in similar way as of dense covariance
+	 * matrix, but direct cholesky factorization of sparse matrices
+	 * could be inefficient. So, this method uses permutation matrix
+	 * for factorization and then permutes back the final samples
+	 * before adding the mean.
+	 *
+	 * @param mean the mean vector
+	 * @param cov the covariance matrix
+	 * @param N number of samples
+	 * @param precision_matrix if true, sample from N(mu,C^-1)
+	 * @return the sample matrix of size \f$N\times dim\f$
+	 */
+	static SGMatrix<float64_t> sample_from_gaussian(SGVector<float64_t> mean,
+	SGSparseMatrix<float64_t> cov, int32_t N=1, bool precision_matrix=false);
+#endif //HAVE_EIGEN3
+
+	/** Magic number for computing lnormal_cdf */
+	static const float64_t ERFC_CASE1;
+
+	/** Magic number for computing lnormal_cdf */
+	static const float64_t ERFC_CASE2;
 
 protected:
 	/** Power series for incomplete beta integral.
@@ -496,6 +664,14 @@ protected:
 	static inline bool greater_equal(float64_t a, float64_t b) { return a>=b; }
 };
 
+/// mean not implemented for complex128_t, returns 0.0 instead
+template <>
+	inline floatmax_t CStatistics::mean<complex128_t>(SGVector<complex128_t> vec)
+	{
+		SG_SNOTIMPLEMENTED
+		return floatmax_t(0.0);
+	}
+
 }
 
 #endif /* __STATISTICS_H_ */
