@@ -17,12 +17,12 @@
 #include <shogun/multiclass/tree/RelaxedTreeUtil.h>
 #include <shogun/multiclass/tree/RelaxedTree.h>
 #include <shogun/kernel/GaussianKernel.h>
-
+#include <shogun/mathematics/Math.h>
 
 using namespace shogun;
 
 CRelaxedTree::CRelaxedTree()
-	:m_max_num_iter(3), m_A(0.5), m_B(5), m_svm_C(1), m_svm_epsilon(0.001), 
+	:m_max_num_iter(3), m_A(0.5), m_B(5), m_svm_C(1), m_svm_epsilon(0.001),
 	m_kernel(NULL), m_feats(NULL), m_machine_for_confusion_matrix(NULL), m_num_classes(0)
 {
 	SG_ADD(&m_max_num_iter, "m_max_num_iter", "max number of iterations in alternating optimization", MS_NOT_AVAILABLE);
@@ -72,7 +72,7 @@ CMulticlassLabels* CRelaxedTree::apply_multiclass(CFeatures* data)
 
 float64_t CRelaxedTree::apply_one(int32_t idx)
 {
-	node_t *node = m_root;
+	bnode_t *node = (bnode_t*) m_root;
 	int32_t klass = -1;
 	while (node != NULL)
 	{
@@ -156,12 +156,12 @@ bool CRelaxedTree::train_machine(CFeatures* data)
 	SG_UNREF(m_root);
 	m_root = train_node(conf_mat, classes);
 
-	std::queue<node_t *> node_q;
-	node_q.push(m_root);
+	std::queue<bnode_t *> node_q;
+	node_q.push((bnode_t*) m_root);
 
 	while (node_q.size() != 0)
 	{
-		node_t *node = node_q.front();
+		bnode_t *node = node_q.front();
 
 		// left node
 		SGVector <int32_t> left_classes(m_num_classes);
@@ -178,7 +178,7 @@ bool CRelaxedTree::train_machine(CFeatures* data)
 
 		if (left_classes.vlen >= 2)
 		{
-			node_t *left_node = train_node(conf_mat, left_classes);
+			bnode_t *left_node = train_node(conf_mat, left_classes);
 			node->left(left_node);
 			node_q.push(left_node);
 		}
@@ -197,7 +197,7 @@ bool CRelaxedTree::train_machine(CFeatures* data)
 
 		if (right_classes.vlen >= 2)
 		{
-			node_t *right_node = train_node(conf_mat, right_classes);
+			bnode_t *right_node = train_node(conf_mat, right_classes);
 			node->right(right_node);
 			node_q.push(right_node);
 		}
@@ -210,7 +210,7 @@ bool CRelaxedTree::train_machine(CFeatures* data)
 	return true;
 }
 
-CRelaxedTree::node_t *CRelaxedTree::train_node(const SGMatrix<float64_t> &conf_mat, SGVector<int32_t> classes)
+CRelaxedTree::bnode_t *CRelaxedTree::train_node(const SGMatrix<float64_t> &conf_mat, SGVector<int32_t> classes)
 {
 	SGVector<int32_t> best_mu;
 	CSVM *best_svm = NULL;
@@ -222,7 +222,7 @@ CRelaxedTree::node_t *CRelaxedTree::train_node(const SGMatrix<float64_t> &conf_m
 		CSVM *svm = new CLibSVM();
 		SG_REF(svm);
 		svm->set_store_model_features(true);
-		
+
 		SGVector<int32_t> mu = train_node_with_initialization(*it, classes, svm);
 		float64_t score = compute_score(mu, svm);
 
@@ -239,7 +239,7 @@ CRelaxedTree::node_t *CRelaxedTree::train_node(const SGMatrix<float64_t> &conf_m
 		}
 	}
 
-	node_t *node = new node_t;
+	bnode_t *node = new bnode_t;
 	SG_REF(node);
 
 	m_machines->push_back(best_svm);
@@ -273,7 +273,7 @@ float64_t CRelaxedTree::compute_score(SGVector<int32_t> mu, CSVM *svm)
 	}
 
 	int32_t totalSV = svm->get_support_vectors().vlen;
-	float64_t score = num_neg/(num_neg+num_pos) * totalSV/num_pos + 
+	float64_t score = num_neg/(num_neg+num_pos) * totalSV/num_pos +
 		num_pos/(num_neg+num_pos)*totalSV/num_neg;
 	return score;
 }
@@ -471,7 +471,7 @@ SGVector<int32_t> CRelaxedTree::color_label_space(CSVM *svm, SGVector<int32_t> c
 	if (npos == 0)
 	{
 		// no positive class
-		index_t min_idx = SGVector<float64_t>::arg_min(xi_pos_class.vector, 1, xi_pos_class.vlen);
+		index_t min_idx = CMath::arg_min(xi_pos_class.vector, 1, xi_pos_class.vlen);
 		mu[min_idx] = 1;
 	}
 
@@ -485,7 +485,7 @@ SGVector<int32_t> CRelaxedTree::color_label_space(CSVM *svm, SGVector<int32_t> c
 	if (nneg == 0)
 	{
 		// no negative class
-		index_t min_idx = SGVector<float64_t>::arg_min(xi_neg_class.vector, 1, xi_neg_class.vlen);
+		index_t min_idx = CMath::arg_min(xi_neg_class.vector, 1, xi_neg_class.vlen);
 		if (mu[min_idx] == 1 && (npos == 0 || npos == 1))
 		{
 			// avoid overwritting the only positive class
@@ -519,11 +519,11 @@ SGVector<int32_t> CRelaxedTree::color_label_space(CSVM *svm, SGVector<int32_t> c
 	return mu;
 }
 
-void CRelaxedTree::enforce_balance_constraints_upper(SGVector<int32_t> &mu, SGVector<float64_t> &delta_neg, 
+void CRelaxedTree::enforce_balance_constraints_upper(SGVector<int32_t> &mu, SGVector<float64_t> &delta_neg,
 		SGVector<float64_t> &delta_pos, int32_t B_prime, SGVector<float64_t>& xi_neg_class)
 {
 	SGVector<index_t> index_zero = mu.find(0);
-	SGVector<index_t> index_pos = mu.find_if(std::bind1st(std::less<int32_t>(), 0)); 
+	SGVector<index_t> index_pos = mu.find_if(std::bind1st(std::less<int32_t>(), 0));
 
 	int32_t num_zero = index_zero.vlen;
 	int32_t num_pos  = index_pos.vlen;
@@ -579,7 +579,7 @@ void CRelaxedTree::enforce_balance_constraints_upper(SGVector<int32_t> &mu, SGVe
 		}
 	}
 
-	SGVector<index_t> sorted_index = S_delta.argsort();
+	SGVector<index_t> sorted_index = CMath::argsort(S_delta);
 	SGVector<float64_t> S_delta_sorted(S_delta.vlen);
 	for (index_t i=0; i < sorted_index.vlen; ++i)
 	{
@@ -708,11 +708,11 @@ void CRelaxedTree::enforce_balance_constraints_upper(SGVector<int32_t> &mu, SGVe
 	}
 }
 
-void CRelaxedTree::enforce_balance_constraints_lower(SGVector<int32_t> &mu, SGVector<float64_t> &delta_neg, 
+void CRelaxedTree::enforce_balance_constraints_lower(SGVector<int32_t> &mu, SGVector<float64_t> &delta_neg,
 		SGVector<float64_t> &delta_pos, int32_t B_prime, SGVector<float64_t>& xi_neg_class)
 {
 	SGVector<index_t> index_zero = mu.find(0);
-	SGVector<index_t> index_neg = mu.find_if(std::bind1st(std::greater<int32_t>(), 0)); 
+	SGVector<index_t> index_neg = mu.find_if(std::bind1st(std::greater<int32_t>(), 0));
 
 	int32_t num_zero = index_zero.vlen;
 	int32_t num_neg  = index_neg.vlen;
@@ -768,7 +768,7 @@ void CRelaxedTree::enforce_balance_constraints_lower(SGVector<int32_t> &mu, SGVe
 		}
 	}
 
-	SGVector<index_t> sorted_index = S_delta.argsort();
+	SGVector<index_t> sorted_index = CMath::argsort(S_delta);
 	SGVector<float64_t> S_delta_sorted(S_delta.vlen);
 	for (index_t i=0; i < sorted_index.vlen; ++i)
 	{
