@@ -10,6 +10,7 @@
 
 #include <shogun/structure/libncbm.h>
 #include <shogun/lib/Time.h>
+#include <shogun/mathematics/Math.h>
 
 #include <shogun/lib/external/libqp.h>
 #include <shogun/multiclass/GMNPLib.h>
@@ -40,7 +41,7 @@ IGNORE_IN_CLASSLIST struct line_search_res
 
 inline static line_search_res zoom
 (
- CStructuredModel* model,
+ CDualLibQPBMSOSVM *machine,
  float64_t lambda,
  float64_t a_lo,
  float64_t a_hi,
@@ -59,7 +60,7 @@ inline static line_search_res zoom
 	line_search_res ls_res;
 	ls_res.solution.resize_vector(initial_solution.vlen);
 	ls_res.gradient.resize_vector(initial_solution.vlen);
-	
+
 	SGVector<float64_t> cur_solution(initial_solution.vlen);
 	cur_solution.zero();
 	SGVector<float64_t> cur_grad(initial_solution.vlen);
@@ -70,7 +71,7 @@ inline static line_search_res zoom
 		float64_t d1 = g_lo+g_hi - 3*(f_lo-f_hi)/(a_lo-a_hi);
 		float64_t d2 = CMath::sqrt(d1*d1 - g_lo*g_hi);
 		float64_t a_j = a_hi -(a_hi-a_lo)*(g_hi+d2-d1)/(g_hi-g_lo+2*d2);
-		
+
 		if (a_lo < a_hi)
 		{
 			if ((a_j < a_lo) || (a_j > a_hi))
@@ -85,14 +86,14 @@ inline static line_search_res zoom
 				a_j = 0.5*(a_lo+a_hi);
 			}
 		}
-		
+
 		cur_solution.add(cur_solution.vector, 1.0,
 				initial_solution.vector, a_j,
 				search_dir.vector, cur_solution.vlen);
 
-		float64_t cur_fval = model->risk(cur_grad.vector, cur_solution.vector);
+		float64_t cur_fval = machine->risk(cur_grad.vector, cur_solution.vector);
 		float64_t cur_reg
-			= 0.5*lambda*cur_solution.dot(cur_solution.vector,
+			= 0.5*lambda*CMath::dot(cur_solution.vector,
 					cur_solution.vector, cur_solution.vlen);
 		cur_fval += cur_reg;
 
@@ -107,12 +108,12 @@ inline static line_search_res zoom
 		{
 			a_hi = a_j;
 			f_hi = cur_fval;
-			g_hi = cur_grad.dot(cur_grad.vector, search_dir.vector, cur_grad.vlen);
+			g_hi = CMath::dot(cur_grad.vector, search_dir.vector, cur_grad.vlen);
 		}
 		else
 		{
 			float64_t cur_lgrad
-				= cur_grad.dot(cur_grad.vector, search_dir.vector, cur_grad.vlen);
+				= CMath::dot(cur_grad.vector, search_dir.vector, cur_grad.vlen);
 
 			if (CMath::abs(cur_lgrad) < -wolfe_c2*init_lgrad)
 			{
@@ -135,7 +136,7 @@ inline static line_search_res zoom
 			f_lo = cur_fval;
 			g_lo = cur_lgrad;
 		}
-		
+
 		if
 			(
 			 (CMath::abs(a_lo - a_hi) <= 0.01*a_lo)
@@ -157,7 +158,7 @@ inline static line_search_res zoom
 
 inline std::vector<line_search_res> line_search_with_strong_wolfe
 (
-		CStructuredModel *model,
+		CDualLibQPBMSOSVM *machine,
 		float64_t lambda,
 		float64_t initial_val,
 		SGVector<float64_t>& initial_solution,
@@ -177,12 +178,12 @@ inline std::vector<line_search_res> line_search_with_strong_wolfe
 	 * subgrad should be: subgrad + (lambda*w)
 	 *
 	 */
-	
+
 	uint32_t iter = 0;
 
 	initial_grad.vec1_plus_scalar_times_vec2(initial_grad.vector, lambda, initial_solution.vector, initial_grad.vlen);
 
-	float64_t initial_lgrad = initial_grad.dot(initial_grad.vector, search_dir.vector, initial_grad.vlen);
+	float64_t initial_lgrad = CMath::dot(initial_grad.vector, search_dir.vector, initial_grad.vlen);
 	float64_t prev_lgrad = initial_lgrad;
 	float64_t prev_fval = initial_val;
 
@@ -196,8 +197,8 @@ inline std::vector<line_search_res> line_search_with_strong_wolfe
 		SGVector<float64_t> cur_subgrad(initial_solution.vlen);
 
 		x.add(x.vector, 1.0, initial_solution.vector, cur_a, search_dir.vector, x.vlen);
-		float64_t cur_fval = model->risk(cur_subgrad.vector, x.vector);
-		float64_t cur_reg = 0.5*lambda*x.dot(x.vector, x.vector, x.vlen);
+		float64_t cur_fval = machine->risk(cur_subgrad.vector, x.vector);
+		float64_t cur_reg = 0.5*lambda*CMath::dot(x.vector, x.vector, x.vlen);
 		cur_fval += cur_reg;
 
 		cur_subgrad.vec1_plus_scalar_times_vec2(cur_subgrad.vector, lambda, x.vector, x.vlen);
@@ -212,7 +213,7 @@ inline std::vector<line_search_res> line_search_with_strong_wolfe
 		}
 
 		float64_t cur_lgrad
-			= cur_subgrad.dot(cur_subgrad.vector, search_dir.vector,
+			= CMath::dot(cur_subgrad.vector, search_dir.vector,
 					cur_subgrad.vlen);
 		if
 			(
@@ -222,7 +223,7 @@ inline std::vector<line_search_res> line_search_with_strong_wolfe
 			)
 		{
 			ret.push_back(
-					zoom(model, lambda, prev_a, cur_a, initial_val,
+					zoom(machine, lambda, prev_a, cur_a, initial_val,
 						initial_solution, search_dir, wolfe_c1, wolfe_c2,
 						initial_lgrad, prev_fval, prev_lgrad, cur_fval, cur_lgrad)
 					);
@@ -244,7 +245,7 @@ inline std::vector<line_search_res> line_search_with_strong_wolfe
 		if (cur_lgrad >= 0)
 		{
 			ret.push_back(
-					zoom(model, lambda, cur_a, prev_a, initial_val,
+					zoom(machine, lambda, cur_a, prev_a, initial_val,
 						initial_solution, search_dir, wolfe_c1, wolfe_c2,
 						initial_lgrad, cur_fval, cur_lgrad, prev_fval, prev_lgrad)
 					);
@@ -271,7 +272,7 @@ inline std::vector<line_search_res> line_search_with_strong_wolfe
 	}
 }
 
-inline void update_H(bmrm_return_value_T& ncbm,
+inline void update_H(BmrmStatistics& ncbm,
 		bmrm_ll* head,
 		bmrm_ll* tail,
 		SGMatrix<float64_t>& H,
@@ -288,7 +289,7 @@ inline void update_H(bmrm_return_value_T& ncbm,
 		float64_t* a_1 = get_cutting_plane(cp_ptr);
 		cp_ptr=cp_ptr->next;
 
-		float64_t dot_val = SGVector<float64_t>::dot(a_2, a_1, w_dim);
+		float64_t dot_val = CMath::dot(a_2, a_1, w_dim);
 
 		H.matrix[LIBBMRM_INDEX(ncbm.nCP, i, maxCP)]
 			= H.matrix[LIBBMRM_INDEX(i, ncbm.nCP, maxCP)]
@@ -296,7 +297,7 @@ inline void update_H(bmrm_return_value_T& ncbm,
 	}
 
 	/* set the diagonal element, i.e. subgrad_i*subgrad_i' */
-	float64_t dot_val = SGVector<float64_t>::dot(a_2, a_2, w_dim);
+	float64_t dot_val = CMath::dot(a_2, a_2, w_dim);
 	H[LIBBMRM_INDEX(ncbm.nCP, ncbm.nCP, maxCP)]=dot_val/lambda;
 
 	diag_H[ncbm.nCP]=H[LIBBMRM_INDEX(ncbm.nCP, ncbm.nCP, maxCP)];
@@ -305,25 +306,28 @@ inline void update_H(bmrm_return_value_T& ncbm,
 }
 
 
-bmrm_return_value_T svm_ncbm_solver(
-		CStructuredModel *model,
-		float64_t        *w,
-		float64_t        TolRel,
-		float64_t        TolAbs,
-		float64_t        _lambda,
-		uint32_t         _BufSize,
-		bool             cleanICP,
-		uint32_t         cleanAfter,
-		bool             is_convex,
-		bool             line_search,
-		bool             verbose
+BmrmStatistics svm_ncbm_solver(
+		CDualLibQPBMSOSVM *machine,
+		float64_t         *w,
+		float64_t         TolRel,
+		float64_t         TolAbs,
+		float64_t         _lambda,
+		uint32_t          _BufSize,
+		bool              cleanICP,
+		uint32_t          cleanAfter,
+		bool              is_convex,
+		bool              line_search,
+		bool              verbose
 		)
 {
-	bmrm_return_value_T ncbm;
+	BmrmStatistics ncbm;
 	libqp_state_T qp_exitflag={0, 0, 0, 0};
+
+	CStructuredModel* model = machine->get_model();
 	int32_t w_dim = model->get_dim();
 
 	maxCPs = _BufSize;
+	BufSize = _BufSize;
 
 	ncbm.nCP=0;
 	ncbm.nIter=0;
@@ -370,10 +374,10 @@ bmrm_return_value_T svm_ncbm_solver(
 	map.set_const(true);
 	ICP_stats icp_stats;
 	icp_stats.maxCPs = maxCPs;
-	icp_stats.ICPcounter = (uint32_t*) LIBBMRM_CALLOC(maxCPs, sizeof(uint32_t));
-	icp_stats.ICPs = (float64_t**) LIBBMRM_CALLOC(maxCPs, sizeof(float64_t*));
-	icp_stats.ACPs = (uint32_t*) LIBBMRM_CALLOC(maxCPs, sizeof(uint32_t));
-	icp_stats.H_buff = (float64_t*) LIBBMRM_CALLOC(maxCPs*maxCPs, sizeof(float64_t));
+	icp_stats.ICPcounter = (uint32_t*) LIBBMRM_CALLOC(maxCPs, uint32_t);
+	icp_stats.ICPs = (float64_t**) LIBBMRM_CALLOC(maxCPs, float64_t*);
+	icp_stats.ACPs = (uint32_t*) LIBBMRM_CALLOC(maxCPs, uint32_t);
+	icp_stats.H_buff = (float64_t*) LIBBMRM_CALLOC(maxCPs*maxCPs,float64_t);
 	if
 	(
 		icp_stats.ICPcounter == NULL || icp_stats.ICPs == NULL
@@ -401,9 +405,9 @@ bmrm_return_value_T svm_ncbm_solver(
 	SGVector<float64_t> cur_w(w_dim);
 	memcpy(cur_w.vector, w, sizeof(float64_t)*w_dim);
 
-	float64_t cur_risk = model->risk(cur_subgrad.vector, cur_w.vector);
+	float64_t cur_risk = machine->risk(cur_subgrad.vector, cur_w.vector);
 	bias[0] = -cur_risk;
-	best_Fp = 0.5*_lambda*cur_w.dot(cur_w.vector, cur_w.vector, cur_w.vlen) + cur_risk;
+	best_Fp = 0.5*_lambda*CMath::dot(cur_w.vector, cur_w.vector, cur_w.vlen) + cur_risk;
 	best_risk = cur_risk;
 	memcpy(best_w.vector, cur_w.vector, sizeof(float64_t)*w_dim);
 	memcpy(best_subgrad.vector, cur_subgrad.vector, sizeof(float64_t)*w_dim);
@@ -502,12 +506,12 @@ bmrm_return_value_T svm_ncbm_solver(
 			{
 				float64_t* a_1 = get_cutting_plane(cp_ptr);
 				cp_ptr = cp_ptr->next;
-				scores[i] = cur_w.dot(cur_w.vector, a_1, w_dim);
+				scores[i] = CMath::dot(cur_w.vector, a_1, w_dim);
 			}
 			scores.vec1_plus_scalar_times_vec2(scores.vector, -1.0, bias.vector, scores.vlen);
 
-			float64_t w_norm = cur_w.dot(cur_w.vector, cur_w.vector, cur_w.vlen);
-			float64_t PO = 0.5*_lambda*w_norm + scores.max(scores.vector, scores.vlen);
+			float64_t w_norm = CMath::dot(cur_w.vector, cur_w.vector, cur_w.vlen);
+			float64_t PO = 0.5*_lambda*w_norm + CMath::max(scores.vector, scores.vlen);
 			float64_t QP_gap = PO - ncbm.Fd;
 
 			SG_SPRINT("%4d: primal:%f dual:%f QP_gap:%f\n", ncbm.nIter, PO, ncbm.Fd, QP_gap)
@@ -523,6 +527,10 @@ bmrm_return_value_T svm_ncbm_solver(
 		if (ncbm.nCP >= maxCPs)
 			ncbm.exitflag = -1;
 
+		// next CP would exceed BufSize/maxCPs
+		if (ncbm.nCP+3 >= maxCPs)
+			ncbm.exitflag=-1;
+
 		tstop=ttime.cur_time_diff(false);
 
 		/* Verbose output */
@@ -531,21 +539,24 @@ bmrm_return_value_T svm_ncbm_solver(
 					ncbm.nIter, tstop-tstart, ncbm.Fp, ncbm.Fd, ncbm.Fp-ncbm.Fd,
 					(ncbm.Fp-ncbm.Fd)/ncbm.Fp, cur_risk, ncbm.nCP, ncbm.nzA, qp_exitflag.exitflag, best_Fp, (best_Fp-ncbm.Fd)/best_Fp);
 
+		if (ncbm.exitflag!=0)
+			break;
+
 		std::vector<line_search_res> wbest_candidates;
 		if (!line_search)
 		{
-			cur_risk = model->risk(cur_subgrad.vector, cur_w.vector);
+			cur_risk = machine->risk(cur_subgrad.vector, cur_w.vector);
 
 			add_cutting_plane(&CPList_tail, map, A.matrix,
 					find_free_idx(map, maxCPs), cur_subgrad.vector, w_dim);
 
-			bias[ncbm.nCP] = cur_w.dot(cur_w.vector, cur_subgrad.vector, cur_w.vlen) - cur_risk;
+			bias[ncbm.nCP] = CMath::dot(cur_w.vector, cur_subgrad.vector, cur_w.vlen) - cur_risk;
 
 			update_H(ncbm, CPList_head, CPList_tail, H, diag_H, _lambda, maxCPs, w_dim);
 
 			// add as a new wbest candidate
 			line_search_res ls;
-			ls.fval = cur_risk+0.5*_lambda*cur_w.dot(cur_w.vector, cur_w.vector, cur_w.vlen);
+			ls.fval = cur_risk+0.5*_lambda*CMath::dot(cur_w.vector, cur_w.vector, cur_w.vlen);
 			ls.solution = cur_w;
 			ls.gradient = cur_subgrad;
 
@@ -574,7 +585,7 @@ bmrm_return_value_T svm_ncbm_solver(
 
 			/* line search */
 			std::vector<line_search_res> ls_res
-				= line_search_with_strong_wolfe(model, _lambda, best_Fp, best_w, best_subgrad, search_dir, astart);
+				= line_search_with_strong_wolfe(machine, _lambda, best_Fp, best_w, best_subgrad, search_dir, astart);
 
 			if (ls_res[0].fval != ls_res[1].fval)
 			{
@@ -584,7 +595,7 @@ bmrm_return_value_T svm_ncbm_solver(
 						find_free_idx(map, maxCPs), ls_res[0].gradient, w_dim);
 
 				bias[ncbm.nCP]
-					= SGVector<float64_t>::dot(ls_res[0].solution.vector, ls_res[0].gradient, w_dim)
+					= CMath::dot(ls_res[0].solution.vector, ls_res[0].gradient, w_dim)
 					- (ls_res[0].fval - ls_res[0].reg);
 
 				update_H(ncbm, CPList_head, CPList_tail, H, diag_H, _lambda, maxCPs, w_dim);
@@ -598,7 +609,7 @@ bmrm_return_value_T svm_ncbm_solver(
 					find_free_idx(map, maxCPs), ls_res[1].gradient.vector, w_dim);
 
 			bias[ncbm.nCP]
-				= ls_res[1].solution.dot(ls_res[1].solution.vector, ls_res[1].gradient.vector, w_dim)
+				= CMath::dot(ls_res[1].solution.vector, ls_res[1].gradient.vector, w_dim)
 				- (ls_res[1].fval - ls_res[1].reg);
 
 			update_H(ncbm, CPList_head, CPList_tail, H, diag_H, _lambda, maxCPs, w_dim);
@@ -607,19 +618,19 @@ bmrm_return_value_T svm_ncbm_solver(
 
 			if ((best_Fp <= ls_res[1].fval) && (astart != 1))
 			{
-				cur_risk = model->risk(cur_subgrad.vector, cur_w.vector);
+				cur_risk = machine->risk(cur_subgrad.vector, cur_w.vector);
 
 				add_cutting_plane(&CPList_tail, map, A.matrix,
 							find_free_idx(map, maxCPs), cur_subgrad.vector, w_dim);
 
 				bias[ncbm.nCP]
-					=  cur_w.dot(cur_w.vector, cur_subgrad.vector, cur_w.vlen) - cur_risk;
+					=  CMath::dot(cur_w.vector, cur_subgrad.vector, cur_w.vlen) - cur_risk;
 
 				update_H(ncbm, CPList_head, CPList_tail, H, diag_H, _lambda, maxCPs, w_dim);
-				
+
 				/* add as a new wbest candidate */
 				line_search_res ls;
-				ls.fval = cur_risk+0.5*_lambda*cur_w.dot(cur_w.vector, cur_w.vector, cur_w.vlen);
+				ls.fval = cur_risk+0.5*_lambda*CMath::dot(cur_w.vector, cur_w.vector, cur_w.vlen);
 				ls.solution = cur_w;
 				ls.gradient = cur_subgrad;
 				SG_SPRINT("%lf\n", ls.fval)
@@ -661,19 +672,19 @@ bmrm_return_value_T svm_ncbm_solver(
 
 				/* conflict */
 				float64_t score
-					= SGVector<float64_t>::dot(best_w.vector,
+					= CMath::dot(best_w.vector,
 							wbest_candidates[i].gradient.vector, w_dim)
 					+ (-1.0*bias[cp_idx]);
 				if (score > best_risk)
 				{
 					float64_t U
 						= best_risk
-						- SGVector<float64_t>::dot(best_w.vector,
+						- CMath::dot(best_w.vector,
 								wbest_candidates[i].gradient.vector, w_dim);
 
 					float64_t L
 						= best_Fp - wbest_candidates[i].reg
-						- SGVector<float64_t>::dot(wbest_candidates[i].solution.vector,
+						- CMath::dot(wbest_candidates[i].solution.vector,
 								wbest_candidates[i].gradient.vector, w_dim);
 
 					if (verbose)
@@ -706,7 +717,7 @@ bmrm_return_value_T svm_ncbm_solver(
 							float64_t* a = get_cutting_plane(cp_ptr);
 							cp_ptr = cp_ptr->next;
 							float64_t dot_val
-								= SGVector<float64_t>::dot(a, wbest_candidates[i].gradient.vector, w_dim);
+								= CMath::dot(a, wbest_candidates[i].gradient.vector, w_dim);
 
 							H.matrix[LIBBMRM_INDEX(cp_idx, j, maxCPs)]
 								= H.matrix[LIBBMRM_INDEX(j, cp_idx, maxCPs)]
@@ -714,13 +725,13 @@ bmrm_return_value_T svm_ncbm_solver(
 						}
 
 						diag_H[LIBBMRM_INDEX(cp_idx, cp_idx, maxCPs)]
-							= SGVector<float64_t>::dot(wbest_candidates[i].gradient.vector,
+							= CMath::dot(wbest_candidates[i].gradient.vector,
 									wbest_candidates[i].gradient.vector, w_dim);
 
 
 						bias[cp_idx]
 							= best_Fp - wbest_candidates[i].reg
-							- SGVector<float64_t>::dot(wbest_candidates[i].solution.vector,
+							- CMath::dot(wbest_candidates[i].solution.vector,
 									wbest_candidates[i].gradient.vector, w_dim);
 
 						if (verbose)
@@ -730,7 +741,7 @@ bmrm_return_value_T svm_ncbm_solver(
 			}
 		}
 
-		/* Inactive Cutting Planes (ICP) removal 
+		/* Inactive Cutting Planes (ICP) removal
 		if (cleanICP)
 		{
 			clean_icp(&icp_stats, ncbm, &CPList_head, &CPList_tail,
@@ -748,6 +759,17 @@ bmrm_return_value_T svm_ncbm_solver(
 	LIBBMRM_FREE(icp_stats.ACPs);
 	LIBBMRM_FREE(icp_stats.H_buff);
 
+        cp_ptr=CPList_head;
+        while(cp_ptr!=NULL)
+        {
+                bmrm_ll * cp_ptr_this=cp_ptr;
+                cp_ptr=cp_ptr->next;
+                LIBBMRM_FREE(cp_ptr_this);
+                cp_ptr_this=NULL;
+        }
+
+	SG_UNREF(model);
+
 	return ncbm;
 }
 
