@@ -11,6 +11,8 @@
 #ifndef libncbm_h
 #define libncbm_h
 
+#include <shogun/lib/config.h>
+
 #include <shogun/lib/common.h>
 #include <shogun/structure/libbmrm.h>
 
@@ -23,8 +25,8 @@ namespace shogun
 	 * min lambda/2 ||w||^2 + R(w)
 	 * where R(w) is a risk funciton of any kind.
 	 */
-	bmrm_return_value_T svm_ncbm_solver(
-			CStructuredModel *model,
+	BmrmStatistics svm_ncbm_solver(
+			CDualLibQPBMSOSVM  *machine,
 			float64_t        *w,
 			float64_t        TolRel,
 			float64_t        TolAbs,
