@@ -11,9 +11,10 @@
  * Implementation of the Proximal Point P-BMRM (p3bm)
  *--------------------------------------------------------------------- */
 
-#include <shogun/structure/libppbm.h>
+#include <shogun/structure/libp3bm.h>
 #include <shogun/lib/external/libqp.h>
 #include <shogun/lib/Time.h>
+ #include <shogun/mathematics/Math.h>
 
 namespace shogun
 {
@@ -21,7 +22,6 @@ static const uint32_t QPSolverMaxIter=0xFFFFFFFF;
 static const float64_t epsilon=0.0;
 
 static float64_t *H, *H2;
-static uint32_t BufSize;
 
 /*----------------------------------------------------------------------
   Returns pointer at i-th column of Hessian matrix.
@@ -31,8 +31,8 @@ static const float64_t *get_col( uint32_t i)
 	return( &H2[ BufSize*i ] );
 }
 
-bmrm_return_value_T svm_p3bm_solver(
-		CStructuredModel* model,
+BmrmStatistics svm_p3bm_solver(
+		CDualLibQPBMSOSVM *machine,
 		float64_t*      W,
 		float64_t       TolRel,
 		float64_t       TolAbs,
@@ -45,7 +45,7 @@ bmrm_return_value_T svm_p3bm_solver(
 		uint32_t        cp_models,
 		bool            verbose)
 {
-	bmrm_return_value_T p3bmrm;
+	BmrmStatistics p3bmrm;
 	libqp_state_T qp_exitflag={0, 0, 0, 0}, qp_exitflag_good={0, 0, 0, 0};
 	float64_t *b, *b2, *beta, *beta_good, *beta_start, *diag_H, *diag_H2;
 	float64_t R, *Rt, **subgrad_t, *A, QPSolverTolRel, *C=NULL;
@@ -60,6 +60,8 @@ bmrm_return_value_T svm_p3bm_solver(
 	bool *map=NULL, tuneAlpha=true, flag=true;
 	bool alphaChanged=false, isThereGoodSolution=false;
 	TMultipleCPinfo **info=NULL;
+	CStructuredModel* model=machine->get_model();
+	CSOSVMHelper* helper = NULL;
 	uint32_t nDim=model->get_dim();
 	uint32_t to=0, N=0, cp_i=0;
 
@@ -92,43 +94,45 @@ bmrm_return_value_T svm_p3bm_solver(
 
 	alpha=0.0;
 
-	H= (float64_t*) LIBBMRM_CALLOC(BufSize*BufSize, sizeof(float64_t));
+	H= (float64_t*) LIBBMRM_CALLOC(BufSize*BufSize, float64_t);
 
-	A= (float64_t*) LIBBMRM_CALLOC(nDim*BufSize, sizeof(float64_t));
+	A= (float64_t*) LIBBMRM_CALLOC(nDim*BufSize, float64_t);
 
-	b= (float64_t*) LIBBMRM_CALLOC(BufSize, sizeof(float64_t));
+	b= (float64_t*) LIBBMRM_CALLOC(BufSize, float64_t);
 
-	beta= (float64_t*) LIBBMRM_CALLOC(BufSize, sizeof(float64_t));
+	beta= (float64_t*) LIBBMRM_CALLOC(BufSize, float64_t);
 
-	subgrad_t= (float64_t**) LIBBMRM_CALLOC(cp_models, sizeof(float64_t*));
+	subgrad_t= (float64_t**) LIBBMRM_CALLOC(cp_models, float64_t*);
 
-	Rt= (float64_t*) LIBBMRM_CALLOC(cp_models, sizeof(float64_t));
+	Rt= (float64_t*) LIBBMRM_CALLOC(cp_models, float64_t);
 
-	diag_H= (float64_t*) LIBBMRM_CALLOC(BufSize, sizeof(float64_t));
+	diag_H= (float64_t*) LIBBMRM_CALLOC(BufSize, float64_t);
 
-	I= (uint32_t*) LIBBMRM_CALLOC(BufSize, sizeof(uint32_t));
+	I= (uint32_t*) LIBBMRM_CALLOC(BufSize, uint32_t);
 
-	cp_list= (bmrm_ll*) LIBBMRM_CALLOC(1, sizeof(bmrm_ll));
+	cp_list= (bmrm_ll*) LIBBMRM_CALLOC(1, bmrm_ll);
 
-	prevW= (float64_t*) LIBBMRM_CALLOC(nDim, sizeof(float64_t));
+	prevW= (float64_t*) LIBBMRM_CALLOC(nDim, float64_t);
 
-	wt= (float64_t*) LIBBMRM_CALLOC(nDim, sizeof(float64_t));
+	wt= (float64_t*) LIBBMRM_CALLOC(nDim, float64_t);
 
-	C= (float64_t*) LIBBMRM_CALLOC(cp_models, sizeof(float64_t));
+	C= (float64_t*) LIBBMRM_CALLOC(cp_models, float64_t);
 
-	S= (uint8_t*) LIBBMRM_CALLOC(cp_models, sizeof(uint8_t));
+	S= (uint8_t*) LIBBMRM_CALLOC(cp_models, uint8_t);
 
-	info= (TMultipleCPinfo**) LIBBMRM_CALLOC(cp_models, sizeof(TMultipleCPinfo*));
+	info= (TMultipleCPinfo**) LIBBMRM_CALLOC(cp_models, TMultipleCPinfo*);
 
-	int32_t num_feats = model->get_features()->get_num_vectors();
+	CFeatures* features = model->get_features();
+	int32_t num_feats = features->get_num_vectors();
+	SG_UNREF(features);
 
 	/* CP cleanup variables */
 	ICP_stats icp_stats;
 	icp_stats.maxCPs = BufSize;
-	icp_stats.ICPcounter= (uint32_t*) LIBBMRM_CALLOC(BufSize, sizeof(uint32_t));
-	icp_stats.ICPs= (float64_t**) LIBBMRM_CALLOC(BufSize, sizeof(float64_t*));
-	icp_stats.ACPs= (uint32_t*) LIBBMRM_CALLOC(BufSize, sizeof(uint32_t));
-	icp_stats.H_buff= (float64_t*) LIBBMRM_CALLOC(BufSize*BufSize, sizeof(float64_t));
+	icp_stats.ICPcounter= (uint32_t*) LIBBMRM_CALLOC(BufSize, uint32_t);
+	icp_stats.ICPs= (float64_t**) LIBBMRM_CALLOC(BufSize, float64_t*);
+	icp_stats.ACPs= (uint32_t*) LIBBMRM_CALLOC(BufSize, uint32_t);
+	icp_stats.H_buff= (float64_t*) LIBBMRM_CALLOC(BufSize*BufSize, float64_t);
 
 	if (H==NULL || A==NULL || b==NULL || beta==NULL || subgrad_t==NULL ||
 			diag_H==NULL || I==NULL || icp_stats.ICPcounter==NULL ||
@@ -149,8 +153,8 @@ bmrm_return_value_T svm_p3bm_solver(
 	{
 		S[p]=1;
 		C[p]=1.0;
-		info[p]=(TMultipleCPinfo*)LIBBMRM_CALLOC(1, sizeof(TMultipleCPinfo));
-		subgrad_t[p]=(float64_t*)LIBBMRM_CALLOC(nDim, sizeof(float64_t));
+		info[p]=(TMultipleCPinfo*)LIBBMRM_CALLOC(1, TMultipleCPinfo);
+		subgrad_t[p]=(float64_t*)LIBBMRM_CALLOC(nDim, float64_t);
 
 		if (subgrad_t[p]==NULL || info[p]==NULL)
 		{
@@ -163,7 +167,7 @@ bmrm_return_value_T svm_p3bm_solver(
 		info[p]->m_N=to-info[p]->m_from;
 	}
 
-	map= (bool*) LIBBMRM_CALLOC(BufSize, sizeof(bool));
+	map= (bool*) LIBBMRM_CALLOC(BufSize, bool);
 
 	if (map==NULL)
 	{
@@ -174,21 +178,21 @@ bmrm_return_value_T svm_p3bm_solver(
 	memset( (bool*) map, true, BufSize);
 
 	/* Temporary buffers */
-	beta_start= (float64_t*) LIBBMRM_CALLOC(BufSize, sizeof(float64_t));
+	beta_start= (float64_t*) LIBBMRM_CALLOC(BufSize, float64_t);
 
-	beta_good= (float64_t*) LIBBMRM_CALLOC(BufSize, sizeof(float64_t));
+	beta_good= (float64_t*) LIBBMRM_CALLOC(BufSize, float64_t);
 
-	b2= (float64_t*) LIBBMRM_CALLOC(BufSize, sizeof(float64_t));
+	b2= (float64_t*) LIBBMRM_CALLOC(BufSize, float64_t);
 
-	diag_H2= (float64_t*) LIBBMRM_CALLOC(BufSize, sizeof(float64_t));
+	diag_H2= (float64_t*) LIBBMRM_CALLOC(BufSize, float64_t);
 
-	H2= (float64_t*) LIBBMRM_CALLOC(BufSize*BufSize, sizeof(float64_t));
+	H2= (float64_t*) LIBBMRM_CALLOC(BufSize*BufSize, float64_t);
 
-	I_start= (uint32_t*) LIBBMRM_CALLOC(BufSize, sizeof(uint32_t));
+	I_start= (uint32_t*) LIBBMRM_CALLOC(BufSize, uint32_t);
 
-	I_good= (uint32_t*) LIBBMRM_CALLOC(BufSize, sizeof(uint32_t));
+	I_good= (uint32_t*) LIBBMRM_CALLOC(BufSize, uint32_t);
 
-	I2= (uint32_t*) LIBBMRM_CALLOC(BufSize, sizeof(uint32_t));
+	I2= (uint32_t*) LIBBMRM_CALLOC(BufSize, uint32_t);
 
 	if (beta_start==NULL || beta_good==NULL || b2==NULL || diag_H2==NULL ||
 			I_start==NULL || I_good==NULL || I2==NULL || H2==NULL)
@@ -202,7 +206,7 @@ bmrm_return_value_T svm_p3bm_solver(
 	p3bmrm.hist_wdist.resize_vector(BufSize);
 
 	/* Iinitial solution */
-	Rt[0] = model->risk(subgrad_t[0], W, info[0]);
+	Rt[0] = machine->risk(subgrad_t[0], W, info[0]);
 
 	p3bmrm.nCP=0;
 	p3bmrm.nIter=0;
@@ -222,8 +226,8 @@ bmrm_return_value_T svm_p3bm_solver(
 
 	for (uint32_t p=1; p<cp_models; ++p)
 	{
-		Rt[p] = model->risk(subgrad_t[p], W, info[p]);
-		b[p]=SGVector<float64_t>::dot(subgrad_t[p], W, nDim) - Rt[p];
+		Rt[p] = machine->risk(subgrad_t[p], W, info[p]);
+		b[p]=CMath::dot(subgrad_t[p], W, nDim) - Rt[p];
 		add_cutting_plane(&CPList_tail, map, A, find_free_idx(map, BufSize), subgrad_t[p], nDim);
 	}
 
@@ -233,7 +237,7 @@ bmrm_return_value_T svm_p3bm_solver(
 	for (uint32_t p=0; p<cp_models; ++p)
 		R+=Rt[p];
 
-	sq_norm_W=SGVector<float64_t>::dot(W, W, nDim);
+	sq_norm_W=CMath::dot(W, W, nDim);
 	sq_norm_Wdiff=0.0;
 
 	for (uint32_t j=0; j<nDim; ++j)
@@ -261,9 +265,12 @@ bmrm_return_value_T svm_p3bm_solver(
 
 	/* Verbose output */
 	if (verbose)
-		SG_SPRINT("%4d: tim=%.3lf, Fp=%lf, Fd=%lf, R=%lf, K=%lf, CPmodels=%d\n",
+		SG_SDEBUG("%4d: tim=%.3lf, Fp=%lf, Fd=%lf, R=%lf, K=%lf, CPmodels=%d\n",
 				p3bmrm.nIter, tstop-tstart, p3bmrm.Fp, p3bmrm.Fd, R, K, cp_models);
 
+	if (verbose)
+		helper = machine->get_helper();
+
 	/* main loop */
 	while (p3bmrm.exitflag==0)
 	{
@@ -281,7 +288,7 @@ bmrm_return_value_T svm_p3bm_solver(
 
 				for (uint32_t p=0; p<cp_models; ++p)
 				{
-					rsum=SGVector<float64_t>::dot(A_1, subgrad_t[p], nDim);
+					rsum=CMath::dot(A_1, subgrad_t[p], nDim);
 
 					H[LIBBMRM_INDEX(p, cp_i, BufSize)]=rsum;
 				}
@@ -299,7 +306,7 @@ bmrm_return_value_T svm_p3bm_solver(
 
 				for (uint32_t p=0; p<cp_models; ++p)
 				{
-					rsum=SGVector<float64_t>::dot(A_1, subgrad_t[p], nDim);
+					rsum=CMath::dot(A_1, subgrad_t[p], nDim);
 
 					H[LIBBMRM_INDEX(p3bmrm.nCP+p, cp_i, BufSize)]=rsum;
 				}
@@ -346,7 +353,7 @@ bmrm_return_value_T svm_p3bm_solver(
 				A_1=get_cutting_plane(cp_ptr);
 				cp_ptr=cp_ptr->next;
 
-				rsum = SGVector<float64_t>::dot(A_1, prevW, nDim);
+				rsum = CMath::dot(A_1, prevW, nDim);
 
 				b2[i]=b[i]-((2*alpha)/(_lambda+2*alpha))*rsum;
 				diag_H2[i]=diag_H[i]/(_lambda+2*alpha);
@@ -405,7 +412,7 @@ bmrm_return_value_T svm_p3bm_solver(
 					A_1=get_cutting_plane(cp_ptr);
 					cp_ptr=cp_ptr->next;
 
-					rsum = SGVector<float64_t>::dot(A_1, prevW, nDim);
+					rsum = CMath::dot(A_1, prevW, nDim);
 
 					b2[i]=b[i]-((2*alpha)/(_lambda+2*alpha))*rsum;
 					diag_H2[i]=diag_H[i]/(_lambda+2*alpha);
@@ -505,7 +512,7 @@ bmrm_return_value_T svm_p3bm_solver(
 				A_1=get_cutting_plane(cp_ptr);
 				cp_ptr=cp_ptr->next;
 
-				rsum = SGVector<float64_t>::dot(A_1, prevW, nDim);
+				rsum = CMath::dot(A_1, prevW, nDim);
 
 				b2[i]=b[i]-((2*alpha)/(_lambda+2*alpha))*rsum;
 				diag_H2[i]=diag_H[i]/(_lambda+2*alpha);
@@ -554,14 +561,14 @@ bmrm_return_value_T svm_p3bm_solver(
 
 		for (uint32_t p=0; p<cp_models; ++p)
 		{
-			Rt[p] = model->risk(subgrad_t[p], W, info[p]);
-			b[p3bmrm.nCP+p] = SGVector<float64_t>::dot(subgrad_t[p], W, nDim) - Rt[p];
+			Rt[p] = machine->risk(subgrad_t[p], W, info[p]);
+			b[p3bmrm.nCP+p] = CMath::dot(subgrad_t[p], W, nDim) - Rt[p];
 			add_cutting_plane(&CPList_tail, map, A, find_free_idx(map, BufSize), subgrad_t[p], nDim);
 			R+=Rt[p];
 		}
 
-		sq_norm_W=SGVector<float64_t>::dot(W, W, nDim);
-		sq_norm_prevW=SGVector<float64_t>::dot(prevW, prevW, nDim);
+		sq_norm_W=CMath::dot(W, W, nDim);
+		sq_norm_prevW=CMath::dot(prevW, prevW, nDim);
 		sq_norm_Wdiff=0.0;
 
 		for (uint32_t j=0; j<nDim; ++j)
@@ -621,7 +628,7 @@ bmrm_return_value_T svm_p3bm_solver(
 
 		/* Verbose output */
 		if (verbose)
-			SG_SPRINT("%4d: tim=%.3lf, Fp=%lf, Fd=%lf, (Fp-Fd)=%lf, (Fp-Fd)/Fp=%lf, R=%lf, nCP=%d, nzA=%d, wdist=%lf, alpha=%lf, qp_cnt=%d, gamma=%lf, tuneAlpha=%d\n",
+			SG_SDEBUG("%4d: tim=%.3lf, Fp=%lf, Fd=%lf, (Fp-Fd)=%lf, (Fp-Fd)/Fp=%lf, R=%lf, nCP=%d, nzA=%d, wdist=%lf, alpha=%lf, qp_cnt=%d, gamma=%lf, tuneAlpha=%d\n",
 					p3bmrm.nIter, tstop-tstart, p3bmrm.Fp, p3bmrm.Fd, p3bmrm.Fp-p3bmrm.Fd,
 					(p3bmrm.Fp-p3bmrm.Fd)/p3bmrm.Fp, R, p3bmrm.nCP, p3bmrm.nzA, wdist, alpha,
 					qp_cnt, gamma, tuneAlpha);
@@ -644,8 +651,27 @@ bmrm_return_value_T svm_p3bm_solver(
 					&CPList_tail, H, diag_H, beta, map,
 					cleanAfter, b, I, cp_models);
 		}
+
+		// next CP would exceed BufSize
+		if (p3bmrm.nCP+1 >= BufSize)
+			p3bmrm.exitflag=-1;
+
+		/* Debug: compute objective and training error */
+		if (verbose)
+		{
+			SGVector<float64_t> w_debug(W, nDim, false);
+			float64_t primal = CSOSVMHelper::primal_objective(w_debug, model, _lambda);
+			float64_t train_error = CSOSVMHelper::average_loss(w_debug, model);
+			helper->add_debug_info(primal, p3bmrm.nIter, train_error);
+		}
 	} /* end of main loop */
 
+	if (verbose)
+	{
+		helper->terminate();
+		SG_UNREF(helper);
+	}
+
 	p3bmrm.hist_Fp.resize_vector(p3bmrm.nIter);
 	p3bmrm.hist_Fd.resize_vector(p3bmrm.nIter);
 	p3bmrm.hist_wdist.resize_vector(p3bmrm.nIter);
@@ -701,6 +727,8 @@ cleanup:
 	LIBBMRM_FREE(subgrad_t);
 	LIBBMRM_FREE(info);
 
+	SG_UNREF(model);
+
 	return(p3bmrm);
 }
 }
